@@ -1,7 +1,7 @@
 /*
- * Miscellaneous routines for Mathomatic.
+ * Standard routines for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -33,7 +33,7 @@ error(str)
 const char	*str;
 {
 	error_str = str;	/* must be a constant string, temporary strings don't work */
-#if	!SILENT
+#if	!LIBRARY
 	set_color(2);		/* set color to red */
 	printf("%s\n", str);
 	default_color();	/* restore to default color */
@@ -83,7 +83,9 @@ const char	*str;	/* constant string to display */
 #endif
 	printf(_("Please report this bug to the maintainers,\n"));
 	printf(_("along with the entry sequence that caused it.\n"));
+#if	!SILENT
 	printf(_("Type \"help bugs\" for info on how to report bugs found in this program.\n"));
+#endif
 	longjmp(jmp_save, 13);	/* Abort the current operation with the critical error number 13. */
 }
 
@@ -107,13 +109,12 @@ check_err()
 		break;
 	case ERANGE:
 		errno = 0;
-		error(_("Overflow error in constant."));
+		error(_("Floating point constant out of range."));
 		longjmp(jmp_save, 2);
 		break;
 	}
 }
 
-#if	UNIX || CYGWIN
 /*
  * Get the screen (window) width and height from the operating system.
  *
@@ -122,6 +123,7 @@ check_err()
 void
 get_screen_size()
 {
+#if	UNIX || CYGWIN
 	struct winsize	ws;
 
 	ws.ws_col = 0;
@@ -134,8 +136,11 @@ get_screen_size()
 			screen_rows = ws.ws_row;
 		}
 	}
-}
+#else
+	screen_columns = STANDARD_SCREEN_COLUMNS;
+	screen_rows = STANDARD_SCREEN_ROWS;
 #endif
+}
 
 /*
  * Allocate the needed global expression storage arrays.
@@ -151,6 +156,8 @@ get_screen_size()
 int
 init_mem()
 {
+	if (n_tokens <= 0)
+		return false;
 	if ((scratch = (token_type *) malloc(((n_tokens * 3) / 2) * sizeof(token_type))) == NULL
 	    || (tes = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
 	    || (tlhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
@@ -164,15 +171,15 @@ init_mem()
 	return true;
 }
 
-#if	false
+#if	LIBRARY
 /*
  * Free the global expression storage arrays.
  * After calling this, memory usage is reset and Mathomatic becomes unusable,
  * so do not call unless finished with using the Mathomatic code.
  *
- * This routine is not needed, because when a program exits,
+ * This routine is usually not needed, because when a program exits,
  * all the memory it allocated is released by the operating system.
- * Inclusion of this routine was requested by Tam Hanna.
+ * Inclusion of this routine was requested by Tam Hanna for Symbian OS.
  */
 void
 free_mem()
@@ -326,16 +333,21 @@ clear_all()
 }
 
 /*
- * Return true if the specified equation space is available, allocating if necessary.
+ * Return true if the specified equation space is available,
+ * zeroing and allocating if necessary.
  */
 int
 alloc_espace(i)
 int	i;	/* equation space number */
 {
-	if (i >= N_EQUATIONS)
+	if (i < 0 || i >= N_EQUATIONS)
 		return false;
+	n_lhs[i] = 0;
+	n_rhs[i] = 0;
 	if (lhs[i] && rhs[i])
 		return true;	/* already allocated */
+	if (lhs[i] || rhs[i])
+		return false;	/* something is wrong */
 	lhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
 	if (lhs[i] == NULL)
 		return false;
@@ -348,6 +360,31 @@ int	i;	/* equation space number */
 	return true;
 }
 
+/*
+ * Allocate all equation spaces up to and including an equation space number,
+ * making sure the specified equation number is valid and usable.
+ *
+ * Returns true if successful.
+ */
+int
+alloc_to_espace(en)
+int	en;	/* equation space number */
+{
+	if (en < 0 || en >= N_EQUATIONS)
+		return false;
+	for (;;) {
+		if (en < n_equations)
+			return true;
+		if (n_equations >= N_EQUATIONS)
+			return false;
+		if (!alloc_espace(n_equations)) {
+			warning(_("Memory is exhausted."));
+			return false;
+		}
+		n_equations++;
+	}
+}
+
 /*
  * Allocate or reuse an empty equation space.
  *
@@ -404,6 +441,7 @@ next_espace()
 
 /*
  * Copy equation space "src" to equation space "dest".
+ * "dest" is overwritten.
  */
 void
 copy_espace(src, dest)
@@ -477,9 +515,33 @@ long	v;	/* standard Mathomatic variable */
 }
 
 /*
- * Substitute every instance of "v" in "equation" with "expression".
+ * Replace all occurrences of variable from_v with to_v in an equation space.
  */
 void
+rename_var_in_es(en, from_v, to_v)
+int	en;	 	/* equation space number */
+long	from_v, to_v;	/* Mathomatic variables */
+{
+	int	i;
+
+	if (n_lhs[en]) {
+		for (i = 0; i < n_lhs[en]; i += 2)
+			if (lhs[en][i].kind == VARIABLE
+			    && lhs[en][i].token.variable == from_v)
+				lhs[en][i].token.variable = to_v;
+		for (i = 0; i < n_rhs[en]; i += 2)
+			if (rhs[en][i].kind == VARIABLE
+			    && rhs[en][i].token.variable == from_v)
+				rhs[en][i].token.variable = to_v;
+	}
+}
+
+/*
+ * Substitute every instance of "v" in "equation" with "expression".
+ *
+ * Return true if something was substituted.
+ */
+int
 subst_var_with_exp(equation, np, expression, len, v)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to equation side length */
@@ -489,9 +551,10 @@ long		v;		/* variable to substitute with expression */
 {
 	int	j, k;
 	int	level;
+	int	substituted = false;
 
 	if (v == 0 || len <= 0)
-		return;
+		return false;
 	for (j = *np - 1; j >= 0; j--) {
 		if (equation[j].kind == VARIABLE && equation[j].token.variable == v) {
 			level = equation[j].level;
@@ -503,12 +566,19 @@ long		v;		/* variable to substitute with expression */
 			blt(&equation[j], expression, len * sizeof(token_type));
 			for (k = j; k < j + len; k++)
 				equation[k].level += level;
+			substituted = true;
 		}
 	}
+	if (substituted) {
+		if (is_integer_var(v) && !is_integer_expr(expression, len)) {
+			warning(_("Substituting integer variable with non-integer expression."));
+		}
+	}
+	return substituted;
 }
 
 /*
- * Return the minimum level encountered in a Mathomatic "expression".
+ * Return the base (minimum) parentheses level encountered in a Mathomatic "expression".
  */
 int
 min_level(expression, n)
@@ -518,12 +588,15 @@ int		n;		/* expression length */
 	int		min1;
 	token_type	*p1, *ep;
 
-	if (n <= 1) {
-		if (n <= 0) {
-			return 1;
-		} else {
-			return expression[0].level;
-		}
+	switch (n) {
+	case 1:
+		return expression[0].level;
+	case 3:
+		return expression[1].level;
+	default:
+		if (n <= 0 || (n & 1) == 0)
+			error_bug("Invalid expression length in call to min_level().");
+		break;
 	}
 	min1 = expression[1].level;
 	ep = &expression[n];
@@ -577,15 +650,17 @@ int		*np;		/* pointer to the returned parsed expression length */
 #if	LIBRARY
 	snprintf(buf, sizeof(buf), "#%+d", pull_number);
 	pull_number++;
-	cp = parse_expr(equation, np, buf);
+	cp = parse_expr(equation, np, buf, true);
+	if (extra_characters(cp))
+		return false;
 	return(cp && *np > 0);
 #else
 	for (;;) {
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
-		cp = parse_expr(equation, np, cp);
-		if (cp) {
+		cp = parse_expr(equation, np, cp, true);
+		if (cp && !extra_characters(cp)) {
 			break;
 		}
 	}
@@ -683,7 +758,7 @@ int	n;		/* maximum size of "string" in bytes */
 	}
 	input_column = strlen(prompt_str);
 #if	READLINE
-	if (readline_enabled && !echo_input) {
+	if (readline_enabled) {
 		cp = readline(prompt_str);
 		if (cp == NULL) {
 			if (!quiet_mode)
@@ -698,9 +773,8 @@ int	n;		/* maximum size of "string" in bytes */
 			free(cp);
 		}
 	} else {
-		if (!echo_input) {
-			printf("%s", prompt_str);
-		}
+		printf("%s", prompt_str);
+		fflush(stdout);
 		if (fgets(string, n, stdin) == NULL) {
 			if (!quiet_mode)
 				printf(_("\nEnd of input.\n"));
@@ -708,9 +782,8 @@ int	n;		/* maximum size of "string" in bytes */
 		}
 	}
 #else
-	if (!echo_input) {
-		printf("%s", prompt_str);
-	}
+	printf("%s", prompt_str);
+	fflush(stdout);
 	if (fgets(string, n, stdin) == NULL) {
 		if (!quiet_mode)
 			printf(_("\nEnd of input.\n"));
@@ -722,7 +795,7 @@ int	n;		/* maximum size of "string" in bytes */
 	if (i >= 0 && string[i] == '\n') {
 		string[i] = '\0';
 	}
-	if ((gfp != stdout && gfp != stderr) || echo_input) {
+	if ((gfp != stdout && gfp != stderr) || (echo_input && !quiet_mode)) {
 		/* Input that is prompted for is now included in the redirected output
 		   of a command to a file, making redirection like logging. */
 		fprintf(gfp, "%s%s\n", prompt_str, string);
@@ -814,7 +887,8 @@ char	*cp;
 /*
  * Process an equation number range given in text string "*cpp".
  * Skip past all spaces and update "*cpp" to point to the next argument if successful.
- * If no equation number or range is given, assume the current equation is wanted.
+ * If no equation number or range is given, assume the current equation is wanted and
+ * don't skip anything.
  *
  * Return true if successful,
  * with the starting equation number in "*ip"
@@ -841,17 +915,22 @@ int	*ip, *jp;
 		} else {
 			*ip = cur_equation;
 		}
-		if (*ip < 0 || *ip >= n_equations) {
-			error(_("Invalid equation number."));
-			return false;
-		}
 		if (*cp != '-') {
-			if (not_defined(*ip)) {
-				return false;
+			if (*cp == '\0' || isspace(*cp)) {
+				if (not_defined(*ip)) {
+					return false;
+				}
+				*jp = *ip;
+				*cpp = skip_space(cp);
+				return true;
+			} else {
+use_current:
+				if (current_not_defined()) {
+					return false;
+				}
+				*jp = *ip = cur_equation;
+				return true;
 			}
-			*jp = *ip;
-			*cpp = skip_space(cp);
-			return true;
 		}
 		(cp)++;
 		if (isdigit(*cp)) {
@@ -859,7 +938,10 @@ int	*ip, *jp;
 		} else {
 			*jp = cur_equation;
 		}
-		if (*jp < 0 || *jp >= n_equations) {
+		if (*cp && !isspace(*cp)) {
+			goto use_current;
+		}
+		if (*ip < 0 || *ip >= N_EQUATIONS || *jp < 0 || *jp >= N_EQUATIONS) {
 			error(_("Invalid equation number."));
 			return false;
 		}
@@ -885,7 +967,7 @@ int	*ip, *jp;
  *
  * Returns true if any non-space characters are encountered before the end of the string
  * and an error message is printed.
- * Otherwise just returns false.
+ * Otherwise just returns false indicating everything is OK.
  */
 int
 extra_characters(cp)
@@ -894,6 +976,7 @@ char	*cp;	/* command line string */
 	if (cp) {
 		cp = skip_space(cp);
 		if (*cp) {
+			printf(_("\nError: \"%s\" not required on input line.\n"), cp);
 			error(_("Extra characters or unrecognized argument."));
 			return true;
 		}
@@ -1119,7 +1202,7 @@ int		n1;	/* expression length */
 {
 	int	i;
 
-	for (i = 0; i < n1; i += 2) {
+	for (i = 0; i < n1; i++) {
 		if (p1[i].kind == CONSTANT && isnan(p1[i].token.constant)) {
 			return true;
 		}
@@ -1138,7 +1221,7 @@ int		n1;	/* expression length */
 {
 	int	i;
 
-	for (i = 0; i < n1; i += 2) {
+	for (i = 0; i < n1; i++) {
 		if (p1[i].kind == VARIABLE && (p1[i].token.variable & VAR_MASK) > SIGN) {
 			return false;		/* not numerical (contains a variable) */
 		}
