@@ -1,7 +1,7 @@
 /*
  * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -59,6 +59,7 @@ do_repeat:
 	}
 	root = strtod(cp, &cp);
 	if ((*cp && !isspace(*cp)) || root < 0.0 || root > MAX_ROOT || fmod(root, 1.0) != 0.0) {
+		error(_("Root invalid or out of range."));
 		printf(_("Root must be a positive integer less than or equal to %.0f.\n"), MAX_ROOT);
 		return false;
 	}
@@ -115,6 +116,9 @@ do_repeat:
 			}
 			fprintf(gfp, "\n");
 #if	!SILENT
+			if (debug_level < 0) {
+				continue;
+			}
 			check = c2;
 			for (d = 1.0; d < root; d += 1.0) {
 				check = complex_mult(check, c2);
@@ -175,7 +179,7 @@ start_over:
 		i += len + 1;
 		r = complex_pow(c, p);
 
-#if	false
+#if	0
 		printf("(%.14g+%.14gi)^(%.14g+%.14gi) = %.14g+%.14gi\n", c.re, c.im, p.re, p.im, r.re, r.im);
 #endif
 
@@ -208,7 +212,7 @@ start_over:
 		goto start_over;
 	}
 	if (modified) {
-		warning(_("Complex number roots approximated."));
+		debug_string(1, _("Complex number roots approximated."));
 	}
 	return modified;
 }
