@@ -1,7 +1,7 @@
 /*
  * Mathomatic symbolic differentiation routines and related commands.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -21,7 +21,7 @@
 
 #include "includes.h"
 
-static int	d_recurse();
+static int d_recurse(token_type *equation, int *np, int loc, int level, long v);
 
 /*
  * Compute the derivative of an equation side, with respect to variable "v",
@@ -350,10 +350,12 @@ char	*cp;
 		if (*cp) {
 			order = decstrtol(cp, &cp);
 		}
-		if (*cp || order <= 0) {
+		if (order <= 0) {
 			error(_("The order must be a positive integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	}
 	if (no_vars(source, *nps, &v)) {
 #if	!SILENT
@@ -382,10 +384,9 @@ char	*cp;
 			printf(_(" %ld times"), order);
 		}
 		if (simplify_flag) {
-			printf(_(" and simplifying...\n"));
-		} else {
-			printf("...\n");
+			printf(_(" and simplifying"));
 		}
+		printf("...\n");
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -458,10 +459,12 @@ char	*cp;
 		if (*cp) {
 			order = decstrtol(cp, &cp);
 		}
-		if (*cp || order <= 0) {
+		if (order <= 0) {
 			error(_("The order must be a positive integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	}
 	if (no_vars(source, n, &v)) {
 		error(_("Current expression contains no variables."));
@@ -587,7 +590,8 @@ char	*cp;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if ((cp = parse_expr(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
+		cp = parse_expr(lhs[our], &our_nlhs, cp, true);
+		if (cp == NULL || extra_characters(cp) || our_nlhs <= 0) {
 			return false;
 		}
 	} else {
@@ -759,7 +763,7 @@ char	*cp;
 		n_lhs[cur_equation] = 1;
 		lhs[cur_equation][0].level = 1;
 		lhs[cur_equation][0].kind = VARIABLE;
-		parse_var(&lhs[cur_equation][0].token.variable, "answer");
+		parse_var(&lhs[cur_equation][0].token.variable, "limit");
 	}
 	if (!solved_equation(cur_equation)) {
 		error(_("The current equation is not solved for a variable."));
@@ -791,7 +795,8 @@ char	*cp;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if ((cp = parse_expr(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
+		cp = parse_expr(tes, &n_tes, cp, true);
+		if (cp == NULL || extra_characters(cp) || n_tes <= 0) {
 			return false;
 		}
 	} else {
@@ -801,12 +806,20 @@ char	*cp;
 			return false;
 		}
 	}
+	simp_loop(tes, &n_tes);
+#if	!SILENT
+	if (debug_level >= 0) {
+		list_var(v, 0);
+		fprintf(gfp, _("Taking the limit as %s goes to "), var_str);
+		list_proc(tes, n_tes, false);
+		fprintf(gfp, "\n");
+	}
+#endif
 /* copy the current equation to a new equation space, then simplify and work on the copy: */
 	copy_espace(cur_equation, i);
 	simpa_side(rhs[i], &n_rhs[i], false, false);
 
 /* see if the limit expression is positive infinity: */
-	simp_loop(tes, &n_tes);
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
 /* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
@@ -824,7 +837,7 @@ char	*cp;
 	}
 
 /* General limit taking, solve for the limit variable: */
-	debug_string(0, "Solving...");
+	debug_string(0, _("Solving..."));
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
