@@ -1,7 +1,7 @@
 /*
  * Mathomatic global variable extern definitions, from file "globals.c".
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -46,6 +46,7 @@ extern int		precision;
 extern int		case_sensitive_flag;
 extern int		factor_int_flag;
 extern int		display2d;
+extern int		fractions_display;
 extern int		approximate_roots;
 extern int		preserve_surds;
 extern int		rationalize_denominators;
@@ -57,9 +58,9 @@ extern int		autosolve;
 extern int		autocalc;
 extern int		autoselect;
 extern char		special_variable_characters[256];
+extern char		plot_prefix[256];
 extern int		integer_coefficients;
 extern int		right_associative_power;
-extern int		negate_highest_precedence;
 extern int		power_starstar;
 extern int		debug_level;
 extern int		domain_check;
@@ -81,12 +82,12 @@ extern char		*prog_name;
 extern char		*var_names[MAX_VAR_NAMES];
 extern char		var_str[MAX_VAR_LEN+80];
 extern char		prompt_str[MAX_PROMPT_LEN];
-extern char		rc_file[PATH_MAX];
+extern char		rc_file[MAX_CMD_LEN];
 
 extern char		*dir_path;
 extern char		*last_history_string;
 extern char		*history_filename;
-extern char		history_filename_storage[PATH_MAX];
+extern char		history_filename_storage[MAX_CMD_LEN];
 
 extern double		unique[];
 extern int		ucnt[];
@@ -96,7 +97,9 @@ extern sign_array_type	sign_array;
 extern FILE		*default_out;
 extern FILE		*gfp;
 extern jmp_buf		jmp_save;
+extern int		eoption;
 extern int		test_mode;
+extern int		demo_mode;
 extern int		quiet_mode;
 extern int		echo_input;
 extern volatile int	abort_flag;
