@@ -1,7 +1,7 @@
 /*
  * Mathomatic symbolic factorizing routines, not polynomial factoring.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -21,13 +21,13 @@
 
 #include "includes.h"
 
-static int	fplus_recurse();
-static int	fplus_sub();
-static int	big_fplus();
-static int	ftimes_recurse();
-static int	ftimes_sub();
-static int	fpower_recurse();
-static int	fpower_sub();
+static int fplus_recurse(token_type *equation, int *np, int loc, int level, long v, double d, int whole_flag, int div_only);
+static int fplus_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level, long v, double d, int whole_flag, int div_only);
+static int big_fplus(token_type *equation, int level, int diff_sign, int sop1, int op1, int op2, int i1, int i2, int b1, int b2, int ai, int aj, int i, int j, int e1, int e2);
+static int ftimes_recurse(token_type *equation, int *np, int loc, int level);
+static int ftimes_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level);
+static int fpower_recurse(token_type *equation, int *np, int loc, int level);
+static int fpower_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level);
 
 /*
  * Factor divides only: (a/c + b/c) -> ((a+b)/c).
@@ -97,10 +97,9 @@ int		div_only;	/* factor only divides */
 {
 	int	modified = false;
 	int	i, j, k;
-	int	op;
+	int	op = 0;
 	int	len1, len2;
 
-	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level) {
 			op = equation[i].token.operatr;
@@ -151,7 +150,7 @@ f_again:
 }
 
 /*
- * Do the factoring of two subexpressions added together.
+ * Do the factoring of two sub-expressions added together.
  *
  * Return true if a transformation was made.
  */
@@ -159,7 +158,7 @@ static int
 fplus_sub(equation, np, loc, i1, n1, i2, n2, level, v, d, whole_flag, div_only)
 token_type	*equation;	/* the entire expression */
 int		*np;		/* pointer to length of the entire expression */
-int		loc;		/* index into the beginning of this additive subexpression */
+int		loc;		/* index into the beginning of this additive sub-expression */
 int		i1, n1, i2, n2;
 int		level;
 long		v;
@@ -221,13 +220,15 @@ f_outer:
 	    || (i - b1 == 1 && equation[b1].kind == CONSTANT && fabs(equation[b1].token.constant) == 1.0)) {
 		goto f_outer;
 	}
-	if (!whole_flag && v && (v != MATCH_ANY)) {
+	if (!whole_flag && (v != MATCH_ANY)) {
 		if (d == 0.0 || d == 1.0) {
-			for (k = b1;; k += 2) {
-				if (k >= i)
-					goto f_outer;
-				if (equation[k].kind == VARIABLE && equation[k].token.variable == v) {
-					break;
+			if (v) {
+				for (k = b1;; k += 2) {
+					if (k >= i)
+						goto f_outer;
+					if (equation[k].kind == VARIABLE && equation[k].token.variable == v) {
+						break;
+					}
 				}
 			}
 		} else {
@@ -238,6 +239,8 @@ f_outer:
 				    && equation[k].level == equation[k+1].level
 				    && equation[k+1].kind == CONSTANT
 				    && equation[k+1].token.constant == d) {
+					if (v == 0)
+						goto factor_this;
 					for (l = k - 1; l >= 0; l--) {
 						if (equation[l].level < equation[k].level)
 							break;
@@ -559,7 +562,7 @@ end_mess:
 }
 
 /*
- * Factor transformation for a more general pair of subexpressions added together
+ * Factor transformation for a more general pair of sub-expressions added together
  * with a common base and any exponent.
  */
 static int
@@ -691,10 +694,9 @@ int		*np, loc, level;
 {
 	int	modified = false;
 	int	i, j, k;
-	int	op;
+	int	op = 0;
 	int	len1, len2;
 
-	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level) {
 			op = equation[i].token.operatr;
@@ -764,7 +766,7 @@ int		*np, loc, i1, n1, i2, n2, level;
 	} else {
 		op1 = equation[i1-1].token.operatr;
 	}
-	if ((n1 == 1 && equation[i1].kind == CONSTANT) || (n2 == 1 && equation[i2].kind == CONSTANT)) {
+	if ((n1 == 1 && equation[i1].kind == CONSTANT) && (n2 == 1 && equation[i2].kind == CONSTANT)) {
 		return false;
 	}
 	both_divide = (op1 == DIVIDE && op2 == DIVIDE);
@@ -894,10 +896,9 @@ int		*np, loc, level;
 {
 	int	modified = false;
 	int	i, j, k;
-	int	op;
+	int	op = 0;
 	int	len1, len2;
 
-	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level) {
 			op = equation[i].token.operatr;
