@@ -1,7 +1,7 @@
 /*
  * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -21,11 +21,11 @@
 
 #include "includes.h"
 
-static void	try_factor();
-static int	fc_recurse();
+static void try_factor(double arg);
+static int fc_recurse(token_type *equation, int *np, int loc, int level, int level_code);
 
 /* The following data is used to factor integers: */
-static double nn, vv;
+static double nn, sqrt_value;
 static double skip_multiples[] = {	/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
@@ -61,22 +61,28 @@ double	value;
 		/* not an integer */
 		return false;
 	}
-	vv = 1.0 + sqrt(fabs(nn));
+	sqrt_value = 1.0 + sqrt(fabs(nn));
 	try_factor(2.0);
 	try_factor(3.0);
 	try_factor(5.0);
 	try_factor(7.0);
 	d = 1.0;
-	while (d <= vv) {
+	while (d <= sqrt_value) {
 		for (i = 0; i < ARR_CNT(skip_multiples); i++) {
 			d += skip_multiples[i];
 			try_factor(d);
 		}
 	}
 	if (nn != 1.0) {
+		if (nn < 0 && nn != -1.0) {
+			try_factor(fabs(nn));
+		}
 		try_factor(nn);
 	}
-	if (value != multiply_out_unique()) {
+	if (uno == 0) {
+		try_factor(1.0);
+	}
+	if (nn != 1.0 || value != multiply_out_unique()) {
 		error_bug("Internal error factoring integers.");
 	}
 	return true;
@@ -91,16 +97,20 @@ try_factor(arg)
 double	arg;
 {
 	while (fmod(nn, arg) == 0.0) {
-		if (uno > 0 && unique[uno-1] == arg) {
+		if (uno > 0 && ucnt[uno-1] > 0 && unique[uno-1] == arg) {
 			ucnt[uno-1]++;
 		} else {
+			while (uno > 0 && ucnt[uno-1] <= 0)
+				uno--;
 			unique[uno] = arg;
-			ucnt[uno] = 1;
-			uno++;
+			ucnt[uno++] = 1;
 		}
 		nn /= arg;
-		vv = 1.0 + sqrt(fabs(nn));
-		if (nn <= 1.0 && nn >= -1.0)
+		if (fmod(nn, 1.0) != 0.0) {
+			error_bug("Turned non-integer in try_factor().");
+		}
+		sqrt_value = 1.0 + sqrt(fabs(nn));
+		if (fabs(nn) <= 1.5 || fabs(arg) <= 1.5)
 			break;
 	}
 }
@@ -291,7 +301,8 @@ int		factor_flag;
  *
  * If "level_code" is 3, nothing is done.
  *
- * Add 4 to "level_code" to always factor out the GCD of rational coefficients.
+ * Add 4 to "level_code" to always factor out the GCD of rational coefficients
+ * to produce all reduced integer coefficients.
  *
  * Return true if equation side was modified.
  */
@@ -438,16 +449,21 @@ int		level_code;
 					d = equation[i].token.constant;
 				}
 			}
-#if	true
+#if	1
 			if (!gcd_flag && minimum >= 1.0) {
 				minimum = 1.0;
 				break;
 			}
 #endif
-			if ((minimum < 1.0 && fmod(d, 1.0) == 0.0) || (fmod(d, minimum) != 0.0)) {
+			if ((minimum < 1.0) && (fmod(d, 1.0) == 0.0)) {
 				minimum = 1.0;
 				break;
 			}
+/* Make sure division by the number to factor out results in an integer: */
+			if (fmod(d, minimum) != 0.0) {
+				minimum = 1.0;	/* result not an integer */
+				break;
+			}
 			i++;
 			for (; i < *np && equation[i].level > level; i += 2)
 				;
