@@ -1,7 +1,7 @@
 /*
  * Mathomatic integration routines and commands.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -21,9 +21,9 @@
 
 #include "includes.h"
 
-static int	integrate_sub();
-static int	laplace_sub();
-static int	inv_laplace_sub();
+static int integrate_sub(token_type *equation, int *np, int loc, int eloc, long v);
+static int laplace_sub(token_type *equation, int *np, int loc, int eloc, long v);
+static int inv_laplace_sub(token_type *equation, int *np, int loc, int eloc, long v);
 
 static int	constant_var_number = 1;	/* makes unique numbers for the constant of integration */
 
@@ -82,7 +82,7 @@ int_dispatch(equation, np, v, func)
 token_type	*equation;	/* pointer to beginning of equation side to integrate */
 int		*np;		/* pointer to length of equation side */
 long		v;		/* integration variable */
-int		(*func)();	/* integration function to call for each term */
+int		(*func)(token_type *equation, int *np, int loc, int eloc, long v);	/* integration function to call for each term */
 {
 	int	i, j;
 
@@ -294,10 +294,12 @@ char	*cp;
 		if (*cp) {
 			integrate_order = strtod(cp, &cp);
 		}
-		if (*cp || integrate_order <= 0 || fmod(integrate_order, 1.0) != 0.0) {
+		if (integrate_order <= 0 || fmod(integrate_order, 1.0) != 0.0) {
 			error(_("The order must be a positive integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
@@ -607,16 +609,16 @@ char	*cp;
 	n1 = *nps;
 	if (inverse_flag) {
 		if (!poly_in_v(dest, n1, v, true) || !int_dispatch(dest, &n1, v, inv_laplace_sub)) {
-			error(_("Inverse Laplace failed."));
+			error(_("Inverse Laplace transformation failed."));
 			return false;
 		}
 	} else {
 		if (!poly_in_v(dest, n1, v, false) || !int_dispatch(dest, &n1, v, laplace_sub)) {
-			error(_("Laplace failed, not a polynomial."));
+			error(_("Laplace transformation failed, not a polynomial."));
 			return false;
 		}
 	}
-#if	true
+#if	1
 	simp_loop(dest, &n1);
 #else
 	simpa_side(dest, &n1, false, false);
@@ -675,10 +677,12 @@ char	*cp;
 		if (*cp) {
 			iterations = decstrtol(cp, &cp);
 		}
-		if (*cp || iterations <= 0 || (iterations % 2) != 0) {
+		if (iterations <= 0 || (iterations % 2) != 0) {
 			error(_("Number of partitions must be a positive, even integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	} else {
 		if (!prompt_var(&v))
 			return false;
