@@ -2,7 +2,7 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -25,15 +25,15 @@
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
 #define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
 
-static int	flist_sub();
-static int	flist_recurse();
+static int flist_sub(token_type *p1, int n, int out_flag, int pos, int *highp, int *lowp);
+static int flist_recurse(token_type *p1, int n, int out_flag, int line, int pos, int cur_level, int *highp, int *lowp);
 
-/* ANSI terminal color code array for 8 color ANSI, we don't use black or white */
-/* because the background may be the same color. */
+/* ANSI terminal color code array for 8 color ANSI; we don't use black or white */
+/* because the background may be the same color, so there are only 6 colors here. */
 static int	carray[] = {
-	32,	/* green (default color) */
-	33,	/* yellow */
-	31,	/* red */
+	32,	/* must be green (default color) */
+	33,	/* must be yellow (for warnings) */
+	31,	/* must be red (for errors) */
 	35,	/* magenta */
 	34,	/* blue */
 	36	/* cyan */
@@ -43,9 +43,9 @@ static int	carray[] = {
 /* Used when HTML output and "set color" and "set bold" options are enabled. */
 /* Good looking with a dark background. */
 static char	*bright_html_carray[] = {
-	"#00FF00",	/* bright green (default color) */
-	"#FFFF00",
-	"#FF0000",
+	"#00FF00",	/* must be bright green (default color) */
+	"#FFFF00",	/* must be bright yellow (for warnings) */
+	"#FF0000",	/* must be bright red (for errors) */
 	"#FF9000",
 	"#FF00FF",
 	"#00FFFF",
@@ -53,9 +53,10 @@ static char	*bright_html_carray[] = {
 };
 
 /* Dim HTML color array for color HTML output. */
+/* Used for HTML output with "set color" and "set no bold" options. */
 /* Good looking with a white background. */
 static char	*html_carray[] = {
-	"green",	/* green (default color) */
+	"green",
 	"olive",
 	"red",
 	"maroon",
@@ -76,17 +77,16 @@ static int	cur_pos;	/* current position in the current line on the screen */
 void
 reset_attr()
 {
+	fflush(NULL);	/* flush all output */
 	if (color_flag && cur_color >= 0) {
 		if (html_flag) {
 			printf("</font>");
 		} else {
 			printf("\033[0m");
 		}
+		fflush(stdout);	/* flush standard output */
 	}
 	cur_color = -1;
-	if (fflush(NULL)) {	/* flush all output */
-		perror("fflush(3) call failed");
-	}
 }
 
 /*
@@ -127,6 +127,7 @@ void
 default_color()
 {
 	set_color(0);
+	fflush(stdout);
 }
 
 /*
@@ -143,7 +144,8 @@ char	*buf;
 		if (buf[j] == '0')
 			continue;
 		if (buf[j] == '.') {
-			buf[j+2] = '\0';
+			if (buf[j+1])
+				buf[j+2] = '\0';
 		} else {
 			break;
 		}
@@ -159,13 +161,13 @@ int
 list1_sub(n, export_flag)
 int	n;		/* equation space number */
 int	export_flag;	/* non-zero for exportable format (readable by other math programs) */
-			/* 1 for Maxima, 2 for other, 3 for Gnuplot */
+			/* 1 for Maxima, 2 for other, 3 for gnuplot, 4 for hexadecimal */
 {
 	int	len = 0;
 
 	if (n_lhs[n] <= 0)
 		return 0;
-	if (!export_flag && !high_prec) {
+	if ((export_flag == 0 || export_flag == 4) && !high_prec) {
 		len += fprintf(gfp, "#%d: ", n + 1);
 	}
 	len += list_proc(lhs[n], n_lhs[n], export_flag);
@@ -176,7 +178,7 @@ int	export_flag;	/* non-zero for exportable format (readable by other math progr
 	if (export_flag == 1) {
 		len += fprintf(gfp, ";");
 	}
-#if	CYGWIN
+#if	CYGWIN || MINGW
 	fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
 #else
 	fprintf(gfp, "\n");
@@ -243,7 +245,7 @@ long	v;	/* Mathomatic variable */
 	char	*cp = NULL;
 	long	l;
 
-	l = (v & VAR_MASK) - VAR_OFFSET;
+	l = (labs(v) & VAR_MASK) - VAR_OFFSET;
 	if (l >= 0 && l < MAX_VAR_NAMES) {
 		cp = var_names[l];
 	}
@@ -259,7 +261,7 @@ long	v;	/* Mathomatic variable */
  *
  * If (lang_code == 0), use standard Mathomatic format.
  * If (lang_code > 0), make variable compatible with the language defined in the enumeration language_list defined in am.h
- * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other, -3 for Gnuplot.
+ * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other, -3 for gnuplot, -4 for hexadecimal.
  */
 int
 list_var(v, lang_code)
@@ -271,7 +273,7 @@ int	lang_code;	/* language code */
 	char		*cp = NULL;
 
 	var_str[0] = '\0';
-	switch (v & VAR_MASK) {
+	switch (labs(v) & VAR_MASK) {
 	case V_NULL:
 		return(strlen(var_str));
 	case SIGN:
@@ -289,6 +291,7 @@ int	lang_code;	/* language code */
 			cp = "%i";
 			break;
 		case 0:
+		case -4:
 			cp = "i#";
 			break;
 		case PYTHON:
@@ -308,6 +311,7 @@ int	lang_code;	/* language code */
 			cp = "%e";
 			break;
 		case 0:
+		case -4:
 			cp = "e#";
 			break;
 		case C:
@@ -330,6 +334,7 @@ int	lang_code;	/* language code */
 			cp = "%pi";
 			break;
 		case 0:
+		case -4:
 			cp = "pi#";
 			break;
 		case C:
@@ -355,8 +360,8 @@ int	lang_code;	/* language code */
 		break;
 	}
 	if (cp) {
-		j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
-		if (j) {
+		j = (labs(v) >> VAR_SHIFT) & SUBSCRIPT_MASK;
+		if (j) {	/* for "sign" variables */
 			snprintf(var_str, sizeof(var_str), "%s%d", cp, j - 1);
 		} else {
 			my_strlcpy(var_str, cp, sizeof(var_str));
@@ -364,9 +369,10 @@ int	lang_code;	/* language code */
 	} else {
 		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
 	}
-	if (from_memory && lang_code) {
+/* Make valid C type variable if exporting or generating code: */
+	if (from_memory && lang_code != 0 && lang_code != -4) {
 		for (j = 0; var_str[j]; j++) {
-			if (var_str[j] != '_' && !isalnum(var_str[j])) {
+			if (strchr("_[]", var_str[j]) == NULL && !isalnum(var_str[j])) {
 				var_str[j] = '_';
 			}
 		}
@@ -385,7 +391,7 @@ list_proc(p1, n, export_flag)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;	/* flag for exportable format (usually false) */
-				/* 1 for Maxima, 2 for other, 3 for Gnuplot */
+				/* 1 for Maxima, 2 for other, 3 for gnuplot, 4 for hexadecimal */
 {
 	return list_string_sub(p1, n, true, NULL, export_flag);
 }
@@ -483,7 +489,7 @@ int		n;		/* length of expression */
 int		outflag;	/* if true, output to gfp */
 char		*string;	/* buffer to save output to or NULL pointer */
 int		export_flag;	/* flag for exportable format (usually false) */
-				/* 1 for Maxima, 2 for other, 3 for Gnuplot */
+				/* 1 for Maxima, 2 for other, 3 for gnuplot, 4 for hexadecimal */
 {
 #define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } if (outflag) { fprintf(gfp, "%s", str); } len += strlen(str); }
 
@@ -494,9 +500,9 @@ int		export_flag;	/* flag for exportable format (usually false) */
 	int	len = 0;
 	char	buf[500], buf2[500];
 	int	export_precision;
-	int	cflag;		/* color flag */
+	int	cflag, power_flag;
 
-	cflag = (outflag && !export_flag);
+	cflag = (outflag && (export_flag == 0 || export_flag == 4));
 	if (string)
 		string[0] = '\0';
 	if (high_prec)
@@ -505,8 +511,27 @@ int		export_flag;	/* flag for exportable format (usually false) */
 		export_precision = DBL_DIG;
 	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
+		power_flag = false;
+#if	1
+		if (export_flag == 0 && !high_prec) {
+			for (j = i - 1; j <= (i + 1); j++) {
+				if ((j - 1) >= 0 && (j + 1) < n
+				    && p1[j].kind == OPERATOR && (p1[j].token.operatr == POWER || p1[j].token.operatr == FACTORIAL)
+				    && p1[j-1].level == p1[j].level
+				    && p1[j+1].level == p1[j].level
+				    && ((j + 2) >= n || p1[j+2].level != (p1[j].level - 1) || (p1[j+2].token.operatr < POWER))
+				    && ((j - 2) < 0 || p1[j-2].level != (p1[j].level - 1) || (p1[j-2].token.operatr < POWER))) {
+					power_flag = true;
+					break;
+				}
+			}
+		}
+#endif
 		j = cur_level - p1[i].level;
-		k = abs(j);
+		if (power_flag)
+			k = abs(j) - 1;
+		else
+			k = abs(j);
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
@@ -525,15 +550,18 @@ int		export_flag;	/* flag for exportable format (usually false) */
 			if (p1[i].token.constant == 0.0) {
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
-			if (export_flag == 3) {
+			if (export_flag == 4) {
+				snprintf(buf, sizeof(buf), "%a", p1[i].token.constant);
+			} else if (export_flag == 3) {
 				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, p1[i].token.constant);
 				trim_zeros(buf);
 			} else if (export_flag || high_prec) {
 				snprintf(buf, sizeof(buf), "%.*g", export_precision, p1[i].token.constant);
 			} else if (finance_option) {
-				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				snprintf(buf, sizeof(buf), "%'.*f", finance_option, p1[i].token.constant);
 			} else {
-				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+				if (p1[i].token.constant < 0.0 && (i + 1) < n && p1[i+1].level == p1[i].level
+				    && (p1[i+1].token.operatr >= POWER)) {
 					snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
 				} else {
 					snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
@@ -634,7 +662,7 @@ int		n;		/* length of expression */
  * Return length of output (number of characters).
  */
 int
-list_c_equation(en, language, int_flag)
+list_code_equation(en, language, int_flag)
 int			en;		/* equation space number */
 enum language_list	language;
 int			int_flag;	/* integer arithmetic flag */
@@ -854,7 +882,7 @@ make_smaller:
 	if (screen_columns && gfp == stdout && width >= screen_columns) {
 		/* output too wide to fit screen, output in single-line format */
 		width = list1_sub(n, false);
-#if	CYGWIN
+#if	CYGWIN || MINGW
 		fprintf(gfp, "\r\n");	/* Be consistent with list1_sub() output. */
 #else
 		fprintf(gfp, "\n");
@@ -936,7 +964,7 @@ int		*highp, *lowp;
 	int	len_div;
 	int	level;
 	int	start_level;
-	int	oflag, html_out;
+	int	oflag, html_out, power_flag;
 	int	len = 0, len1, len2;
 	int	high, low;
 	char	buf[500];
@@ -983,12 +1011,32 @@ check_again:
 		}
 	}
 	for (i = ii; i < n; i++) {
+		power_flag = false;
 		if (i == stop_at) {
+			if (div_loc < 0) {
+				error_bug("Bug in flist_recurse().");
+			}
 			j = cur_level - p1[div_loc].level;
 			k = abs(j) - 1;
 		} else {
+#if	1
+			for (j = i - 1; j <= (i + 1); j++) {
+				if ((j - 1) >= ii && (j + 1) < n
+				    && p1[j].kind == OPERATOR && (p1[j].token.operatr == POWER || p1[j].token.operatr == FACTORIAL)
+				    && p1[j-1].level == p1[j].level
+				    && p1[j+1].level == p1[j].level
+				    && ((j + 2) >= n || p1[j+2].level != (p1[j].level - 1) || (p1[j+2].token.operatr < POWER))
+				    && ((j - 2) < ii || p1[j-2].level != (p1[j].level - 1) || (p1[j-2].token.operatr < POWER))) {
+					power_flag = true;
+					break;
+				}
+			}
+#endif
 			j = cur_level - p1[i].level;
-			k = abs(j);
+			if (power_flag)
+				k = abs(j) - 1;
+			else
+				k = abs(j);
 		}
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
@@ -1029,10 +1077,11 @@ check_again:
 			if (low < *lowp)
 				*lowp = low;
 			if (oflag) {
+				/* display fraction bar */
 				set_color(level-1);
 				for (j = 0; j < len_div; j++) {
 					if (html_out) {
-						fprintf(gfp, "&minus;");
+						fprintf(gfp, "&ndash;");
 					} else {
 						fprintf(gfp, "-");
 					}
@@ -1056,18 +1105,27 @@ check_again:
 			if (p1[i].token.constant == 0.0) {
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
-			if (p1[i].token.constant == -1.0 && (i == 0 || p1[i-1].level < p1[i].level)
+			if (html_out && isinf(p1[i].token.constant)) {
+				if (p1[i].token.constant < 0) {
+					snprintf(buf, sizeof(buf), "(-&infin;)");
+					len += 4;
+				} else {
+					snprintf(buf, sizeof(buf), "&infin;");
+					len += 1;
+				}
+			} else if (p1[i].token.constant == -1.0 && (i == 0 || p1[i-1].level < p1[i].level)
 			    && (i + 1) < n && p1[i].level == p1[i+1].level && p1[i+1].token.operatr == TIMES) {
 				i++;
 				len += snprintf(buf, sizeof(buf), "-");
 			} else if (finance_option) {
 				if (p1[i].token.constant < 0.0) {
-					len += snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
+					len += snprintf(buf, sizeof(buf), "(%'.*f)", finance_option, p1[i].token.constant);
 				} else {
-					len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+					len += snprintf(buf, sizeof(buf), "%'.*f", finance_option, p1[i].token.constant);
 				}
 			} else {
-				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+				if (p1[i].token.constant < 0.0 && (i + 1) < n && p1[i+1].level == p1[i].level
+				    && (p1[i+1].token.operatr >= POWER)) {
 					len += snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
 				} else {
 					len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
@@ -1117,23 +1175,14 @@ check_again:
 				len += strlen(cp);
 				break;
 			case POWER:
-#if	false
+#if	0
 				if (html_out
-				    && p1[i-1].level == p1[i].level
 				    && p1[i+1].level == p1[i].level
 				    && p1[i+1].kind == CONSTANT) {
-					if (p1[i+1].token.constant == 2.0) {
-						cp = "&sup2;";
-						len++;
-						i++;
-						break;
-					}
-					if (p1[i+1].token.constant == 3.0) {
-						cp = "&sup3;";
-						len++;
-						i++;
-						break;
-					}
+					len += (snprintf(buf, sizeof(buf), "<sup>%.*g</sup>", precision, p1[i+1].token.constant) - 11);
+					cp = buf;
+					i++;
+					break;
 				}
 #endif
 				if (power_starstar) {
