@@ -2,7 +2,7 @@
  * This file contains main() and the startup code for Mathomatic,
  * which is a small computer algebra system written entirely in C.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -56,36 +56,52 @@
  * to adjust memory usage.
  */
 
-#if	!LIBRARY	/* This comments out this whole file if using as symbolic math library. */
+#if	!LIBRARY	/* This comments out this whole file if compiling as the symbolic math library. */
 
 #include "includes.h"
-#if	TRAP_FPERRORS	/* Do not use this define!  Currently does not work. */
-#include <fenv.h>
+#include <getopt.h>
+
+void
+display_startup_message(fp)
+FILE	*fp;
+{
+#if	SECURE
+	fprintf(fp, _("Secure "));
+#else
+	if (security_level >= 2)
+		fprintf(fp, _("Secure "));
 #endif
+	fprintf(fp, "Mathomatic version %s (www.mathomatic.org)\n", VERSION);
+	fprintf(fp, "Copyright (C) 1987-2011 George Gesslein II.\n");
+	fprintf(fp, _("%d equation spaces available, %ld kilobytes per equation space.\n"),
+	    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
+}
 
 /*
  * Display invocation usage info.
  */
 void
-usage()
+usage(fp)
+FILE	*fp;
 {
-	printf(_("\nMathomatic computer algebra system version %s\n\n"), VERSION);
-	printf(_("Usage: %s [ options ] [ input_files ]\n\n"), prog_name);
-	printf(_("Options:\n"));
-	printf(_("  -b            Enable bold color mode.\n"));
-	printf(_("  -c            Toggle color mode.\n"));
-	printf(_("  -h            Display this help and exit.\n"));
-	printf(_("  -m number     Specify a memory size multiplier.\n"));
-	printf(_("  -q            Set quiet mode (don't display prompts).\n"));
-	printf(_("  -r            Disable readline.\n"));
-	printf(_("  -s level      Set enforced security level for session.\n"));
-	printf(_("  -t            Set test mode.\n"));
-	printf(_("  -u            Set unbuffered output.\n"));
-	printf(_("  -v            Display version information, then exit.\n"));
-	printf(_("  -w            Wide output mode, sets unlimited width.\n"));
-	printf(_("  -x            Enable HTML/XHTML output mode.\n"));
-	printf(_("\nPlease refer to the Mathomatic man page for more information.\n"));
-	printf(_("The man page is viewed by typing \"man mathomatic\" in shell.\n"));
+	fprintf(fp, _("Mathomatic computer algebra system\n"));
+	fprintf(fp, _("Usage: %s [ options ] [ input_files or input ]\n\n"), prog_name);
+	fprintf(fp, _("Options:\n"));
+	fprintf(fp, _("  -b            Enable bold color mode.\n"));
+	fprintf(fp, _("  -c            Toggle color mode.\n"));
+	fprintf(fp, _("  -e            Process expressions and commands on the command line.\n"));
+	fprintf(fp, _("  -h            Display this help and exit.\n"));
+	fprintf(fp, _("  -m number     Specify a memory size multiplier.\n"));
+	fprintf(fp, _("  -q            Set quiet mode (don't display prompts).\n"));
+	fprintf(fp, _("  -r            Disable readline.\n"));
+	fprintf(fp, _("  -s level      Set enforced security level for session.\n"));
+	fprintf(fp, _("  -t            Set test mode.\n"));
+	fprintf(fp, _("  -u            Set unbuffered output with input echo.\n"));
+	fprintf(fp, _("  -v            Display version information, then exit.\n"));
+	fprintf(fp, _("  -w            Wide output mode, sets unlimited width.\n"));
+	fprintf(fp, _("  -x            Enable HTML/XHTML output mode.\n"));
+	fprintf(fp, _("\nPlease refer to the Mathomatic man page for more information.\n"));
+	fprintf(fp, _("The man page is viewed by typing \"man mathomatic\" in shell.\n"));
 }
 
 int
@@ -93,38 +109,42 @@ main(argc, argv)
 int	argc;
 char	**argv;
 {
-	extern char	*optarg;	/* set by getopt(3) */
-	extern int	optind;
-
 	int		i;
 	char		*cp = NULL;
 	double		numerator, denominator;
-	double		new_size;
+	double		new_size = 0;
 	int		coption = false, boption = false, wide_flag = false;
+	int		exit_value = 0;
 
 #if	I18N
 	/* Initialize internationalization so that messages are in the right language. */
 	setlocale(LC_ALL, "");
-	textdomain(prog_name);
 	bindtextdomain(prog_name, LOCALEDIR);
+	textdomain(prog_name);
 #endif
 
-#if	CYGWIN
+#if	CYGWIN || MINGW
 	dir_path = strdup(dirname_win(argv[0]));	/* set dir_path to this executable's directory */
 #endif
 	/* initialize the global variables */
 	init_gvars();
 	default_out = stdout;	/* set default_out to any file you want to redirect output to */
 	gfp = default_out;
+	get_screen_size();
 
 	/* process command line options */
-	while ((i = getopt(argc, argv, "s:bqrtchuvwxm:")) >= 0) {
+	while ((i = getopt(argc, argv, "s:bqrtdchuvwxm:e")) >= 0) {
 		switch (i) {
 		case 's':
 #if	SECURE
 			fprintf(stderr, _("%s: Compiled for maximum security, therefore no need for setting security level.\n"), prog_name);
 #else
-			security_level = atoi(optarg);
+			if (optarg)
+				security_level = strtod(optarg, &cp);
+			if (optarg == NULL || cp == NULL || cp == optarg || *cp) {
+				fprintf(stderr, _("%s: Error in setting security level.\n"), prog_name);
+				exit(2);
+			}
 #endif
 			break;
 		case 'w':
@@ -141,8 +161,9 @@ char	**argv;
 			wide_flag = true;
 			break;
 		case 'm':
-			new_size = strtod(optarg, &cp) * DEFAULT_N_TOKENS;
-			if (cp == NULL || *cp || new_size <= 0 || new_size >= (INT_MAX / 3)) {
+			if (optarg)
+				new_size = strtod(optarg, &cp) * DEFAULT_N_TOKENS;
+			if (optarg == NULL || cp == NULL || *cp || new_size <= 0 || new_size >= (INT_MAX / sizeof(token_type))) {
 				fprintf(stderr, _("%s: Invalid memory size multiplier specified.\n"), prog_name);
 				exit(2);
 			}
@@ -159,24 +180,31 @@ char	**argv;
 			wide_flag = true;
 			test_mode = true;
 			break;
+		case 'd':
+			demo_mode = true;
+			break;
 		case 'u':
+			echo_input = true;
 			setbuf(stdout, NULL);	/* make output unbuffered */
 			setbuf(stderr, NULL);
 			break;
 		case 'h':
-			usage();
+			usage(stdout);
 			exit(0);
 		case 'v':
 			version_report();
 			exit(0);
+		case 'e':
+			eoption = true;
+			autoselect = false;
+			break;
 		default:
-			usage();
+			usage(stdout);
 			exit(2);
 		}
 	}
-	if (n_tokens < 100) {
-		fprintf(stderr, _("%s: Expression array size too small.\n"), prog_name);
-		exit(2);
+	if (n_tokens < 100 || n_tokens >= (INT_MAX / sizeof(token_type))) {
+		fprintf(stderr, _("%s: Standard expression array size %d out of range!\n"), prog_name, n_tokens);
 	}
 	if (!init_mem()) {
 		fprintf(stderr, _("%s: Not enough memory.\n"), prog_name);
@@ -185,44 +213,39 @@ char	**argv;
 #if	READLINE
 	if (readline_enabled) {	/* readline_enabled flag must not change after this */
 		if ((cp = getenv("HOME")) && prog_name) {
+#if	CYGWIN || MINGW
+			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/%s_history", cp, prog_name);
+#else
 			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/.%s_history", cp, prog_name);
+#endif
 			history_filename = history_filename_storage;
 		}
-		rl_initialize();		/* initialize readline */
 		using_history();		/* initialize readline history */
+		rl_initialize();		/* initialize readline */
 		stifle_history(500);		/* maximum of 500 entries */
 		rl_inhibit_completion = true;	/* turn off readline file name completion */
-#if	!NO_COLOR && !LIBRARY	/* not 100% tested and this might confuse the user with the -c toggle */
+#if	0	/* not 100% tested and this might confuse the user with the -c toggle */
 		if (!html_flag) {		/* If doing ANSI color: */
-			color_flag = (tigetnum("colors") >= 8);	/* autoset color output mode. */
+			color_flag = (tigetnum("colors") >= 8);	/* autoset color output mode.  Requires ncurses. */
 		}
 #endif
 #if	!SECURE
-		read_history(history_filename);	/* restore readline history of previous session */
+		if (security_level <= 3) {
+			read_history(history_filename);	/* restore readline history of previous session */
+		}
 #endif
 	}
-#endif
-#if	UNIX || CYGWIN
-	get_screen_size();
 #endif
 	if (html_flag) {
 		printf("<pre>\n");
 	}
-	if (!test_mode && !quiet_mode) {
-#if	SECURE
-		printf(_("Secure "));
-#else
-		if (security_level >= 2)
-			printf(_("Secure "));
-#endif
-		printf("Mathomatic version %s (www.mathomatic.org)\n", VERSION);
-		printf("Copyright (C) 1987-2010 George Gesslein II.\n");
-		printf(_("%d equation spaces available, %ld kilobytes per equation space.\n"),
-		    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
+	if (!test_mode && !quiet_mode && !eoption) {
+		display_startup_message(stdout);
 	}
+	fflush(stdout);
 #if	!SECURE
 	/* read the user options initialization file */
-	if (!test_mode && !load_rc()) {
+	if (security_level <= 3 && !test_mode && !demo_mode && !load_rc()) {
 		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n"), prog_name, rc_file);
 	}
 #endif
@@ -239,15 +262,28 @@ char	**argv;
 	}
 	if (test_mode) {
 		color_flag = false;
-	}
-	if (!quiet_mode && color_flag) {
-		printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
-		if (!boption) {
-			printf(_("; disable with the -c option or \"set no color\".\n"));
+	} else if (!quiet_mode && !eoption) {
+		if (color_flag) {
+			printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
+#if	!SECURE
+			if (!boption && security_level == 0)
+				printf(_("; disable with the -c option or \"set no color\".\n"));
+			else
+#endif
+				printf(".\n");
 		} else {
-			printf(".\n");
+#if	!MINGW
+			printf(_("Color mode turned off"));
+#if	!SECURE
+			if (security_level == 0)
+				printf(_("; enable with the -c option or \"set color\".\n"));
+			else
+#endif
+				printf(".\n");
+#endif
 		}
 	}
+	fflush(stdout);
 	if ((i = setjmp(jmp_save)) != 0) {
 		/* for error handling */
 		clean_up();
@@ -269,21 +305,43 @@ char	**argv;
 			fprintf(stderr, _("%s: Cannot convert any floating point values to fractions.\n"), prog_name);
 			fprintf(stderr, _("Roots will not work properly.\n"));
 		}
-#if	!SECURE
-		/* read in files specified on the command line, exit if error */
-		for (i = optind; i < argc && argv[i]; i++) {
-			if (strcmp(argv[i], "-") == 0) {
-				main_io_loop();
-			} else if (!read_file(argv[i])) {
-				fprintf(stderr, _("Read of file \"%s\" failed.\n"), argv[i]);
-				exit_program(1);
-			}
+		if (max_memory_usage() <= 0) {
+			fprintf(stderr, _("%s: Calculated maximum memory usage overflows a long integer!\n"), prog_name);
 		}
+		if (eoption) {
+			/* process expressions and commands from the command line */
+			for (i = optind; i < argc && argv[i]; i++) {
+				if (!display_process(argv[i])) {
+					exit_value = 1;
+				}
+			}
+		} else {
+#if	SECURE
+			if (!html_flag && !quiet_mode && !test_mode)
+				printf(_("Anything done here is temporary.\n"));
+			if (optind < argc) {
+				warning(_("Command-line argument ignored."));
+			}
+#else
+			if (!html_flag && !quiet_mode && !test_mode)
+				printf(_("Anything done here is temporary, unless it is saved or redirected.\n"));
+			/* read in files specified on the command line, exit if error */
+			for (i = optind; i < argc && argv[i]; i++) {
+				if (strcmp(argv[i], "-") == 0) {
+					main_io_loop();
+				} else if (!read_file(argv[i])) {
+					fflush(NULL);	/* flush all output */
+					fprintf(stderr, _("Read of file \"%s\" failed.\n"), argv[i]);
+					exit_program(1);
+				}
+			}
 #endif
+		}
 	}
-	main_io_loop();		/* main input/output loop */
-	exit_program(0);	/* exit Mathomatic, doesn't return */
-	return(0);		/* so the compiler doesn't complain */
+	if (!eoption)
+		main_io_loop();		/* main input/output loop */
+	exit_program(exit_value);	/* exit Mathomatic, doesn't return */
+	return(exit_value);		/* so the compiler doesn't complain */
 }
 
 /*
@@ -315,18 +373,14 @@ set_signals()
 
 	if (signal(SIGFPE, fphandler) == SIG_ERR)
 		rv = false;
-#if	TRAP_FPERRORS
-	if (feenableexcept(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW) < 0)
-		rv = false;
-#endif
 	if (signal(SIGINT, inthandler) == SIG_ERR)
 		rv = false;
-	if (signal(SIGQUIT, inthandler) == SIG_ERR)
-		rv = false;
 	if (signal(SIGTERM, exithandler) == SIG_ERR)
 		rv = false;
+#if	!MINGW
 	if (signal(SIGHUP, exithandler) == SIG_ERR)
 		rv = false;
+#endif
 #if	UNIX || CYGWIN
 	if (signal(SIGWINCH, resizehandler) == SIG_ERR)
 		rv = false;
@@ -339,7 +393,7 @@ set_signals()
 	return rv;
 }
 
-#if	CYGWIN
+#if	CYGWIN || MINGW
 /*
  * dirname(3) function for Microsoft Windows.
  * dirname(3) strips the non-directory suffix from a filename.
@@ -382,8 +436,12 @@ load_rc()
 		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, ".mathomaticrc");
 		fp = fopen(rc_file, "r");
 	}
-#if	CYGWIN
-	if (fp == NULL) {
+#if	CYGWIN || MINGW
+	if (fp == NULL && cp) {
+		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, "mathomatic.rc");
+		fp = fopen(rc_file, "r");
+	}
+	if (fp == NULL && dir_path) {
 		snprintf(rc_file, sizeof(rc_file), "%s/%s", dir_path, "mathomatic.rc");
 		fp = fopen(rc_file, "r");
 	}
@@ -391,12 +449,18 @@ load_rc()
 	if (fp == NULL) {
 		return true;
 	}
+	if (!quiet_mode && !eoption) {
+		printf(_("Loading startup set options from \"%s\".\n"), rc_file);
+	}
 	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
 		set_error_level(cp);
 		if (!set_options(cp))
 			rv = false;
 	}
-	fclose(fp);
+	if (fclose(fp)) {
+		rv = false;
+		perror(rc_file);
+	}
 	return rv;
 }
 #endif
@@ -404,19 +468,12 @@ load_rc()
 /*
  * Floating point exception handler.
  * Floating point exceptions are currently ignored.
- *
- * I do not know the proper function calls that should be here to make the TRAP_FPERRORS define work.
- * Currently it goes in an endless loop when an FP exception occurs and TRAP_FPERRORS is true,
- * because when this function returns, the exception immediately happens again.
- * Therefore, do not define TRAP_FPERRORS!
  */
 void
 fphandler(sig)
 int	sig;
 {
-#if	TRAP_FPERRORS
-	warning(_("Floating point exception."));
-#endif
+/*	warning(_("Floating point exception.")); */
 }
 
 /*
@@ -491,14 +548,13 @@ int	exit_value;	/* zero if OK, non-zero indicates error return */
 		printf("</pre>\n");
 	}
 #if	READLINE && !SECURE
-	if (readline_enabled) {
-		if (write_history(history_filename)) {	/* save readline history */
-			fprintf(stderr, _("%s: Unable to save readline history into file \"%s\".\n"), prog_name, history_filename);
-		}
+	if (readline_enabled && security_level <= 3) {
+		write_history(history_filename);	/* save readline history */
 	}
 #endif
-	if (exit_value == 0 && !quiet_mode && !html_flag)
-		printf(_("Thank you for using Mathomatic!\n"));
+	if (exit_value == 0 && !quiet_mode && !eoption && !html_flag) {
+		printf(_("ByeBye!! from Mathomatic.\n"));
+	}
 	exit(exit_value);
 }
 #endif
