@@ -1,7 +1,7 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -21,8 +21,6 @@
 
 #include "includes.h"
 
-#define	INFINITY_NAME	"inf"	/* set this to the name of the infinity constant as displayed by printf(3) */
-
 int	point_flag;		/* point to location of parse error if true */
 
 /*
@@ -34,25 +32,28 @@ char	*cp;
 {
 	if (cp) {
 		for (; *cp; cp++) {
-			if (isupper(*cp))
+			if (isascii(*cp) && isupper(*cp))
 				*cp = tolower(*cp);
 		}
 	}
 }
 
 /*
- * Display an up arrow pointing to the error under the input string.
+ * Display an up arrow pointing to the error, if appropriate.
+ * The up arrow is positioned under the input string,
+ * followed by the error message, which must be a constant string.
  */
 void
 put_up_arrow(cnt, cp)
 int	cnt;	/* position of error, relative to "input_column" */
-char	*cp;	/* error message */
+char	*cp;	/* error message (constant ASCII string) */
 {
 #if	!SILENT
 	int	 i;
 
-	if (isatty(0) && point_flag && !html_flag) {
-		for (i = 0; i < (input_column + cnt); i++) {
+	cnt += input_column;
+	if (!quiet_mode && point_flag && (screen_columns == 0 || cnt < screen_columns)) {
+		for (i = 0; i < cnt; i++) {
 			printf(" ");
 		}
 		printf("^ ");
@@ -71,6 +72,27 @@ int	ch;
 	return(ch == '_' || (ch && strchr(special_variable_characters, ch)) || isalpha(ch));
 }
 
+/*
+ * Return true if character (ch) is the beginning of a Mathomatic operator.
+ */
+int
+is_mathomatic_operator(ch)
+int	ch;
+{
+	switch (ch) {
+	case '!':
+	case '*':
+	case '^':
+	case '/':
+	case '%':
+	case '+':
+	case '-':
+	case '=':
+		return true;
+	}
+	return false;
+}
+
 /*
  * Parenthesize an operator.
  */
@@ -82,12 +104,12 @@ int		i;	/* location of operator to parenthesize in expression */
 {
 	int	j;
 	int	level;
+	int	skip_negate;
 
-#if	true
-	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1) {
+	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1 || p1[i].kind != OPERATOR) {
 		error_bug("Internal error in arguments to binary_parenthesize().");
 	}
-#endif
+	skip_negate = (p1[i].token.operatr != NEGATE);
 	level = p1[i].level++;
 	if (p1[i-1].level++ > level) {
 		for (j = i - 2; j >= 0; j--) {
@@ -96,18 +118,23 @@ int		i;	/* location of operator to parenthesize in expression */
 			p1[j].level++;
 		}
 	}
+finish:
 	if (p1[i+1].level++ > level) {
 		for (j = i + 2; j < n; j++) {
 			if (p1[j].level <= level)
 				break;
 			p1[j].level++;
 		}
+	} else if (skip_negate && p1[i+1].level > level && i + 3 < n && p1[i+2].level == level
+	    && p1[i+2].token.operatr == NEGATE) {
+		p1[i+2].level++;
+		i += 2;
+		goto finish;
 	}
 }
 
 /*
- * Handle and remove the special high priority NEGATE operator.
- * To make negate the same priority as times/divide, don't parenthesize here.
+ * Handle and remove the special NEGATE operator.
  */
 void
 handle_negate(equation, np)
@@ -118,12 +145,10 @@ int		*np;
 
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == NEGATE) {
-			/* change negate operator to -1 times the operand: */
+			/* parenthesize it first: */
+			binary_parenthesize(equation, *np, i);
+			/* finish changing negate operator to -1.0 times the operand: */
 			equation[i].token.operatr = TIMES;
-			if (negate_highest_precedence) {
-				/* make negate top priority by parenthesizing it first: */
-				binary_parenthesize(equation, *np, i);
-			}	/* otherwise negate has same priority as times/divide */
 		}
 	}
 }
@@ -145,7 +170,7 @@ int		*np;		/* pointer to expression length */
 
 /* Higher priority (precedence) operators need to be parenthesized first: */
 	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == FACTORIAL) {
+		if (equation[i].token.operatr >= FACTORIAL) {
 			binary_parenthesize(equation, *np, i);
 		}
 	}
@@ -174,6 +199,7 @@ int		*np;		/* pointer to expression length */
 			break;
 		}
 	}
+	handle_negate(equation, np); /* Just make this the first called function here to make negate highest priority. */
 }
 
 /*
@@ -192,7 +218,8 @@ int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
 int		allow_space;	/* if false, any space characters terminate parsing */
 {
-	int		n = 0;			/* position in equation[] */
+	int		i;
+	int		n = 0, old_n;		/* position in equation[] */
 	int		cur_level = 1;		/* current level of parentheses */
 	int		operand = false;	/* flip-flop between operand and operator */
 	char		*cp_start, *cp1;
@@ -211,7 +238,8 @@ int		allow_space;	/* if false, any space characters terminate parsing */
 		case '(':
 		case '{':
 			if (operand) {
-#if	true
+#if	1
+/* Allow things like (x)(y) and x{y} by defaulting to the times operator.  The result is x*y. */
 				operand = false;
 				equation[n].level = cur_level;
 				equation[n].kind = OPERATOR;
@@ -236,6 +264,7 @@ int		allow_space;	/* if false, any space characters terminate parsing */
 			continue;
 		case ' ':
 		case '\t':
+		case '\f':
 			if (allow_space)
 				continue;
 		case '=':
@@ -243,6 +272,14 @@ int		allow_space;	/* if false, any space characters terminate parsing */
 		case '\0':
 		case '\n':
 			goto p_out;	/* terminator encountered */
+		case '\r':
+			continue;
+		case '\033':
+			if (cp[1] == '[' || cp[1] == 'O') {
+				error(_("Cursor or function key string encountered, unable to correct."));
+				return(NULL);
+			}
+			continue;
 		}
 		operand = !operand;
 		switch (*cp) {
@@ -338,7 +375,13 @@ parse_power:
 			break;
 		case '%':
 			if (operand) {
+#if	1
+/* Allow %e, %i, and %pi by ignoring % in the wrong place. */
+				operand = false;
+				break;
+#else
 				goto syntax_error;
+#endif
 			}
 			equation[n].level = cur_level;
 			equation[n].kind = OPERATOR;
@@ -387,7 +430,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (*cp == '-' && (!negate_highest_precedence || !(isdigit(cp[1]) || cp[1] == '.'))) {
+			if (*cp == '-') {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = -1.0;
 				equation[n].level = cur_level;
@@ -415,6 +458,48 @@ parse_power:
 			n++;
 			cp--;
 			break;
+		case '#':
+			if (!operand) {
+				goto syntax_error;
+			}
+			cp++;
+			switch (*cp) {
+			case '+':
+			case '-':
+				i = decstrtol(cp, &cp1);
+				i = cur_equation + i;
+				break;
+			default:
+				i = decstrtol(cp, &cp1) - 1;
+				break;
+			}
+			if (cp == cp1) {
+				put_up_arrow(cp - cp_start, _("Error parsing equation space number after #."));
+				return NULL;
+			}
+			if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
+				put_up_arrow(cp - cp_start, _("No expression available in # specified equation space."));
+				return NULL;
+			}
+			cp = cp1 - 1;
+			old_n = n;
+			if (n_rhs[i]) {
+				n += n_rhs[i];
+				if (n > n_tokens) {
+					error_huge();
+				}
+				blt(&equation[old_n], rhs[i], n_rhs[i] * sizeof(token_type));
+			} else {
+				n += n_lhs[i];
+				if (n > n_tokens) {
+					error_huge();
+				}
+				blt(&equation[old_n], lhs[i], n_lhs[i] * sizeof(token_type));
+			}
+			for (; old_n < n; old_n++) {
+				equation[old_n].level += cur_level;
+			}
+			break;
 		default:
 			if (!isvarchar(*cp)) {
 				put_up_arrow(cp - cp_start, _("Unrecognized character."));
@@ -462,7 +547,6 @@ p_out:
 		cp++;
 	*np = n;
 	if (n) {
-		handle_negate(equation, np);
 		give_priority(equation, np);
 		organize(equation, np);
 	}
@@ -476,6 +560,8 @@ syntax_error:
 
 /*
  * Parse an equation string into equation space "n".
+ * The result will not necessarily be an equation,
+ * because this can parse any expression or equation.
  *
  * Returns the new string position, or NULL if error.
  * Currently, there can be no more to parse in the string when this returns.
@@ -485,41 +571,42 @@ parse_equation(n, cp)
 int	n;	/* equation space number */
 char	*cp;	/* pointer to the beginning of the equation character string */
 {
-	if (cp == NULL)
-		return NULL;
-	if (!case_sensitive_flag) {
-		str_tolower(cp);
-	}
-	if ((cp = parse_section(lhs[n], &n_lhs[n], cp, true)) != NULL) {
-		if ((cp = parse_section(rhs[n], &n_rhs[n], cp, true)) != NULL) {
-			if (extra_characters(cp))
-				return NULL;
-			return cp;
+	if ((cp = parse_expr(lhs[n], &n_lhs[n], cp, true)) != NULL) {
+		if ((cp = parse_expr(rhs[n], &n_rhs[n], cp, true)) != NULL) {
+			if (!extra_characters(cp))
+				return cp;
 		}
 	}
+	n_lhs[n] = 0;
+	n_rhs[n] = 0;
 	return NULL;
 }
 
 /*
- * Parse an expression with equation space pull if the line starts with "#" followed by an equation number.
+ * Parse an expression (not an equation) string, with equation space pull
+ * if the string starts with "#" followed by an equation number.
  *
  * Returns the new string position, or NULL if error.
- * Currently, there can be no more to parse in the string when this returns.
  */
 char *
-parse_expr(equation, np, cp)
+parse_expr(equation, np, cp, allow_space)
 token_type	*equation;	/* where the parsed expression is stored (equation side) */
 int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
+int		allow_space;
 {
+#if	0	/* Now serving "#" parsing in parse_section(). */
 	int	i;
-	char	*cp1, *cp2;
+	char	*cp1, *cp2, *cp_start;
 
+	cp_start = cp;
+#endif
 	if (cp == NULL)
 		return NULL;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
+#if	0
 	cp1 = skip_space(cp);
 	if (*cp1 == '#') {
 		cp1++;
@@ -533,12 +620,12 @@ char		*cp;		/* string to parse */
 			i = decstrtol(cp1, &cp2) - 1;
 			break;
 		}
-		if (cp1 == cp2 || *cp2) {
-			error(_("Error parsing equation space number."));
+		if (cp1 == cp2) {
+			put_up_arrow(cp1 - cp_start, _("Error parsing equation space number after #."));
 			return NULL;
 		}
 		if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
-			error(_("No expression available in specified equation space."));
+			put_up_arrow(cp1 - cp_start, _("No expression available in # specified equation space."));
 			return NULL;
 		}
 		if (n_rhs[i]) {
@@ -552,12 +639,13 @@ char		*cp;		/* string to parse */
 		list_proc(equation, *np, false);
 		fprintf(gfp, "\n");
 #endif
+		while (*cp2 == '=')
+			cp2++;
+		input_column += (cp2 - cp_start);
 		return cp2;
 	}
-	if ((cp = parse_section(equation, np, cp, true)) != NULL) {
-		if (extra_characters(cp))
-			return NULL;
-	}
+#endif
+	cp = parse_section(equation, np, cp, allow_space);
 	return cp;
 }
 
@@ -603,12 +691,13 @@ char	*cp;
 		buf[i++] = *cp1++;
 	}
 	buf[i] = '\0';
+
+	if (strcasecmp(buf, NAN_NAME) == 0) {
+		warning(_("Attempt to enter NaN (Not a Number); Converted to variable."));
+	}
 	if (strcasecmp(buf, INFINITY_NAME) == 0) {
 		error(_("Infinity cannot be used as a variable."));
 		return(NULL);
-	} else if (strcasecmp(buf, "nan") == 0) {
-		error(_("NaN (Not a Number) cannot be directly entered."));
-		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
 	} else {
@@ -679,6 +768,7 @@ char	*cp;
 		*vp = vtmp;
 		return cp1;
 	}
+/* for "sign" variables: */
 	if (isdigit(*cp1)) {
 		j = strtol(cp1, &cp1, 10);
 		if (j < 0 || j > MAX_SUBSCRIPT) {
@@ -689,6 +779,8 @@ char	*cp;
 			sign_array[j+1] = true;
 		}
 		vtmp += ((long) (j + 1)) << VAR_SHIFT;
+	} else if (vtmp == SIGN) {
+		sign_array[0] = true;
 	}
 	*vp = vtmp;
 	return cp1;
@@ -720,10 +812,23 @@ set_error_level(cp)
 char	*cp;	/* input string */
 {
 	char	*cp1;
+	int	len;
 
 	point_flag = true;
 /* handle comments, newlines, and the DOS EOF character (control-Z) by truncating the string where found */
-	cp1 = strpbrk(cp, ";\n\r\032");
+	cp1 = cp;
+	while ((cp1 = strpbrk(cp1, ";\n\r\032"))) {
+		if (cp1 != cp && *(cp1 - 1) == '\\') {
+			if (*cp1 == ';') {
+/* skip backslash escaped semicolon */
+				point_flag = false;
+				len = strlen(cp1);
+				blt(cp1 - 1, cp1, len + 1);
+				continue;
+			}
+		}
+		break;
+	}
 	if (cp1) {
 		*cp1 = '\0';
 	}
@@ -745,12 +850,12 @@ var_is_const(v, dp)
 long	v;
 double	*dp;
 {
-	switch (v) {
-	case V_E:
+	if (v == V_E) {
 		if (dp)
 			*dp = M_E;
 		return true;
-	case V_PI:
+	}
+	if (v == V_PI) {
 		if (dp)
 			*dp = M_PI;
 		return true;
