@@ -18,7 +18,7 @@
  * They are not separated out from the main variable of the polynomial.
  * Maximum simplification of all expressions is not possible unless everything is generalized.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -55,9 +55,15 @@ int		n_quotient;			/* length of expression in quotient[] */
 token_type	gcd_divisor[DIVISOR_SIZE];	/* static expression storage area for polynomial GCD routine */
 int		len_d;				/* length of expression in gcd_divisor[] */
 
-static int	do_gcd(), pf_recurse(), pf_sub(), mod_recurse(), polydiv_recurse(), pdiv_recurse(), poly_div_sub();
-static int	find_highest_count();
-static void	save_factors();
+static int pf_recurse(token_type *equation, int *np, int loc, int level, int do_repeat);
+static int pf_sub(token_type *equation, int *np, int loc, int len, int level, int do_repeat);
+static void save_factors(token_type *equation, int *np, int loc1, int len, int level);
+static int do_gcd(long *vp);
+static int mod_recurse(token_type *equation, int *np, int loc, int level);
+static int polydiv_recurse(token_type *equation, int *np, int loc, int level);
+static int pdiv_recurse(token_type *equation, int *np, int loc, int level, int code);
+static int poly_div_sub(token_type *d1, int len1, token_type *d2, int len2, long *vp);
+static int find_highest_count(token_type *p1, int n1, token_type *p2, int n2, long *vp1);
 
 /*
  * Compare function for qsort(3).
@@ -66,6 +72,13 @@ static int
 vcmp(p1, p2)
 sort_type	*p1, *p2;
 {
+	if (p2->count == p1->count) {
+		if (p1->v < p2->v)
+			return -1;
+		if (p1->v == p2->v)
+			return 0;
+		return 1;
+	}
 	return(p2->count - p1->count);
 }
 
@@ -75,7 +88,7 @@ sort_type	*p1, *p2;
  * The coefficient (c) and exponent (d) may be any expression,
  * as long as it doesn't contain the variable v.
  */
-static int
+int
 poly_in_v_sub(p1, n, v, allow_divides)
 token_type	*p1;		/* expression pointer */
 int		n;		/* expression length */
@@ -154,9 +167,13 @@ int		allow_divides;	/* allow variable to be right of a divide (negative exponent
 }
 
 /*
- * Factor polynomials by calling pf_sub() for every additive subexpression.
- * Factors repeated factor polynomials (like (x+1)^5)
- * and symbolic factor polynomials (like (x+a)*(x+b)).
+ * Factor polynomials by calling pf_sub() for every additive sub-expression.
+ * Factors repeated factor polynomials (like (x+1)^5) if "do_repeat".
+ * And factors polynomials with symbolic factors (like (x+a)*(x+b)).
+ *
+ * Does not factor polynomials with different numeric factors (like (x+1)*(x+2)).
+ * Because of floating point round-off error,
+ * this routine does not succeed at factoring very large polynomials.
  *
  * Return true if equation side was modified (factored).
  */
@@ -177,21 +194,21 @@ int		do_repeat;
 {
 	int	modified = false;
 	int	i;
-	int	op;
-	int	len;
+	int	count = 0, level_count = 0;
 
-	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-		if (equation[i].level == level) {
-			op = equation[i].token.operatr;
+		switch (equation[i].token.operatr) {
+		case PLUS:
+		case MINUS:
+			count++;
+			if (equation[i].level == level) {
+				level_count++;
+			}
 		}
 	}
-	len = i - loc;
-	switch (op) {
-	case PLUS:
-	case MINUS:
-		modified = pf_sub(equation, np, loc, len, level, do_repeat);
-		break;
+	if (level_count && count > 1) {	/* so we don't factor expressions with only one additive operator */
+		/* try to factor the sub-expression */
+		modified = pf_sub(equation, np, loc, i - loc, level, do_repeat);
 	}
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
@@ -227,8 +244,7 @@ int		do_repeat;	/* factor repeated factors flag */
 	int		i, j, k;
 	long		v = 0, v1, last_v;
 	int		len_first = 0;
-	int		loc1;
-	int		loc2, len2 = 0;
+	int		loc1, loc2, len2 = 0;
 	int		loct, lent;
 	int		count;
 	jmp_buf		save_save;
@@ -312,7 +328,7 @@ int		do_repeat;	/* factor repeated factors flag */
 		if (cnt <= 1)
 			goto skip_factor;
 		if (v == 0) {
-#if	true
+#if	1
 			goto skip_factor;
 #else
 /* no polynomial variables found, try differentiating with respect to each variable until one works */
@@ -359,9 +375,9 @@ int		do_repeat;	/* factor repeated factors flag */
 		tlhs[n_tlhs].token.variable = v;
 		n_tlhs++;
 		uf_simp(tlhs, &n_tlhs);
-		if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, v) == 0)
+		if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, v) <= 0)
 			break;
-		if (!level1_plus_count(tlhs, n_tlhs))
+		if (level1_plus_count(tlhs, n_tlhs) == 0)
 			break;
 		save_factors(equation, np, loc1, len, level);
 		loc1 += n_tlhs + 1;
@@ -422,9 +438,9 @@ next_v:
 				fprintf(gfp, "\n");
 			}
 #endif
-			if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L) == 0)
+			if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L) <= 0)
 				goto next_v;
-			if (!level1_plus_count(tlhs, n_tlhs))
+			if (level1_plus_count(tlhs, n_tlhs) == 0)
 				goto next_v;
 			debug_string(1, "Symbolic polynomial factored.");
 			save_factors(equation, np, loc1, len, level);
@@ -448,7 +464,7 @@ skip_factor:
 			loct = loc;
 			lent = len_first;
 		}
-		if (poly_gcd(&equation[loc1], len, &equation[loct], lent, v)) {
+		if (poly_gcd(&equation[loc1], len, &equation[loct], lent, v) > 0) {
 			save_factors(equation, np, loc1, len, level);
 			loc1 += n_tlhs + 1;
 			len = n_trhs;
@@ -457,7 +473,7 @@ skip_factor:
 			loc1 = loc2;
 			len = len2;
 		}
-		if (poly_gcd(&equation[loc], len_first, &equation[loc1], len, 0L)) {
+		if (poly_gcd(&equation[loc], len_first, &equation[loc1], len, 0L) > 0) {
 			save_factors(equation, np, loc, len_first, level);
 		}
 	}
@@ -489,7 +505,8 @@ int		*np, loc1, len, level;
 }
 
 /*
- * Remove level 1 trivial factors and divides from tlhs.
+ * Remove level 1 trivial factors and divides from tlhs[].
+ * Makes it simpler and look a lot nicer.
  *
  * Return true if result is a level 1 additive expression.
  * If this returns false, nothing is removed.
@@ -504,7 +521,7 @@ remove_factors()
 	debug_string(3, "Entering remove_factors() with:");
 	side_debug(3, tlhs, n_tlhs);
 	do {
-		simp_ssub(tlhs, &n_tlhs, 0L, 1.0, false, true, 4 /* was 0 */);
+		simp_ssub(tlhs, &n_tlhs, 0L, 1.0, false, true, 4);
 	} while (uf_power(tlhs, &n_tlhs));
 	for (i = 1, j = 0, k = 0;; i += 2) {
 		if (i >= n_tlhs) {
@@ -558,14 +575,16 @@ remove_factors()
 
 /*
  * This is the Euclidean GCD algorithm applied to polynomials.
- * It needs to be made multivariate.
+ * It needs to be made multivariate by making it recursive.
  *
- * Return the number of iterations (divisions), if successful.
- * Return 0 on failure.
+ * Return the number of iterations (divisions), if successful,
+ * with the polynomial GCD result in gcd_divisor[] and len_d.
+ * Else return 0 or negative on failure.  If 0, might work if operands switched,
+ * If a negative number, switching operands won't work either.
  */
 static int
 do_gcd(vp)
-long		*vp;
+long		*vp;	/* polynomial base variable pointer */
 {
 	int	i;
 	int	count;
@@ -574,12 +593,14 @@ long		*vp;
 		switch (poly_div(trhs, n_trhs, gcd_divisor, len_d, vp)) {
 		case 0:
 			/* divide failed */
-			return 0;
+			return(1 - count);
 		case 2:
 			/* Total success!  Remainder is zero. */
+			debug_string(2, "Found raw polynomial GCD:");
+			side_debug(2, gcd_divisor, len_d);
 			return count;
 		}
-		/* swap trhs[] (remainder) and gcd_divisor[] */
+		/* Do the Euclidean shuffle: swap trhs[] (remainder) and gcd_divisor[] */
 		if (len_d > n_tokens || n_trhs > DIVISOR_SIZE)
 			return 0;
 		blt(scratch, trhs, n_trhs * sizeof(token_type));
@@ -593,22 +614,28 @@ long		*vp;
 }
 
 /*
- * Compute the polynomial Greatest Common Divisor of the expressions in "larger" and "smaller".
- * This GCD routine is used for polynomial factoring, etc.
+ * Compute the simplified and normalized polynomial Greatest Common Divisor
+ * of the expressions in "larger" and "smaller".
+ * This polynomial GCD routine is used for polynomial factoring, etc.
  *
- * Return true if successful.
+ * Return a positive integer if successful.
  * Return the GCD in trhs[].
  * Return larger/GCD in tlhs[].
+ * The results are unfactored and simplified.
  */
 int
 poly_gcd(larger, llen, smaller, slen, v)
 token_type	*larger;	/* larger polynomial */
+int		llen;		/* larger polynomial length */
 token_type	*smaller;	/* smaller polynomial */
-int		llen, slen;
-long		v;
+int		slen;		/* smaller polynomial length */
+long		v;		/* polynomial base variable */
 {
 	int		count;
 
+	debug_string(3, "Entering poly_gcd():");
+	side_debug(3, larger, llen);
+	side_debug(3, smaller, slen);
 	if (llen > n_tokens || slen > min(ARR_CNT(gcd_divisor), n_tokens))
 		return 0;
 	blt(trhs, larger, llen * sizeof(token_type));
@@ -622,7 +649,7 @@ long		v;
 	blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 	len_d = n_tlhs;
 	count = do_gcd(&v);
-	if (count == 0)
+	if (count <= 0)
 		return 0;
 	if (count > 1) {
 		if (len_d > n_tokens)
@@ -652,23 +679,47 @@ long		v;
 
 /*
  * Compute the polynomial Greatest Common Divisor of the expressions in "larger" and "smaller".
- * This GCD routine is used by division simplifiers.
+ * This polynomial GCD routine is used by the division simplifiers.
  *
- * Return true if successful.
+ * Return a positive integer if successful.
  * Return larger/GCD in tlhs[].
  * Return smaller/GCD in trhs[].
  */
 int
-poly2_gcd(larger, llen, smaller, slen, v)
+poly2_gcd(larger, llen, smaller, slen, v, require_additive)
 token_type	*larger;	/* larger polynomial */
+int		llen;		/* larger polynomial length */
 token_type	*smaller;	/* smaller polynomial */
-int		llen, slen;
-long		v;
+int		slen;		/* smaller polynomial length */
+long		v;		/* polynomial base variable */
+int		require_additive;	/* require the GCD to contain addition or subtraction */
 {
 	int		i;
-	jmp_buf		save_save;
 	int		count;
+#if	1
+	jmp_buf		save_save;
+#endif
 
+	if (require_additive) {
+/* Require an additive operator in both polynomials to continue. */
+		count = 0;
+		for (i = 1; i < llen; i += 2) {
+			if (larger[i].token.operatr == PLUS || larger[i].token.operatr == MINUS) {
+				count++;
+				break;
+			}
+		}
+		if (count == 0)
+			return 0;
+		count = 0;
+		for (i = 1; i < slen; i += 2) {
+			if (smaller[i].token.operatr == PLUS || smaller[i].token.operatr == MINUS) {
+				count++;
+			}
+		}
+		if (count == 0 /* || count > 200 */)
+			return 0;
+	}
 	debug_string(3, "Entering poly2_gcd():");
 	side_debug(3, larger, llen);
 	side_debug(3, smaller, slen);
@@ -678,29 +729,32 @@ long		v;
 	n_trhs = llen;
 	blt(tlhs, smaller, slen * sizeof(token_type));
 	n_tlhs = slen;
-#if	true
-	blt(save_save, jmp_save, sizeof(jmp_save));
-	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
-		blt(jmp_save, save_save, sizeof(jmp_save));
-		if (i == 13) {	/* critical error code */
-			longjmp(jmp_save, i);
+#if	1
+	if (require_additive) {
+/* Require a level 1 additive operator in the divisor. */
+		blt(save_save, jmp_save, sizeof(jmp_save));
+		if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+			blt(jmp_save, save_save, sizeof(jmp_save));
+			if (i == 13) {	/* critical error code */
+				longjmp(jmp_save, i);
+			}
+			return 0;
 		}
-		return 0;
+		uf_simp(tlhs, &n_tlhs);
+		blt(jmp_save, save_save, sizeof(jmp_save));
+		if (level1_plus_count(tlhs, n_tlhs) == 0)
+			return 0;
 	}
-	uf_simp(tlhs, &n_tlhs);
-	blt(jmp_save, save_save, sizeof(jmp_save));
 #endif
-	if (!level1_plus_count(tlhs, n_tlhs))
-		return 0;
 	if (n_tlhs > ARR_CNT(gcd_divisor))
 		return 0;
 	blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 	len_d = n_tlhs;
 	count = do_gcd(&v);
-	if (count == 0)
-		return 0;
+	if (count <= 0)
+		return count;
 	if (count > 1) {
-		if (!level1_plus_count(gcd_divisor, len_d))
+		if (require_additive && level1_plus_count(gcd_divisor, len_d) == 0)
 			return 0;
 		if (poly_div(smaller, slen, gcd_divisor, len_d, &v) != 2) {
 			debug_string(1, "Polynomial GCD found, but smaller divide failed in poly2_gcd().");
@@ -768,6 +822,7 @@ token_type	*p1;	/* expression pointer */
 int		n;	/* length of expression */
 {
 	int	i;
+	long	v;
 
 	for (i = 0; i < n; i++) {
 		switch (p1[i].kind) {
@@ -780,8 +835,8 @@ int		n;	/* length of expression */
 				return false;
 			break;
 		case VARIABLE:
-			if (!is_integer_var(p1[i].token.variable)
-			    && (p1[i].token.variable & VAR_MASK) != SIGN)
+			v = labs(p1[i].token.variable);
+			if (!is_integer_var(v) && (v & VAR_MASK) != SIGN)
 				return false;
 			break;
 		}
@@ -790,7 +845,9 @@ int		n;	/* length of expression */
 }
 
 /*
- * This routine is a modulus operator (%) simplifier for equation sides.
+ * This routine is the modulus operator (%) simplifier for equation sides.
+ * Using "integer" variables will allow more simplification here.
+ * Globals tlhs[] and trhs[] are wiped out by the polynomial division.
  *
  * Return true if expression was modified.
  */
@@ -813,7 +870,6 @@ int		*np, loc, level;
 	int	op, last_op2;
 	int	len1, len2, len3;
 	int	diff_sign;
-	long	v;		/* Mathomatic variable */
 
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
@@ -828,124 +884,123 @@ int		*np, loc, level;
 	if (modified)	/* make sure the deepest levels are completed, first */
 		return true;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-		if (equation[i].level == level && equation[i].token.operatr == MODULUS) {
-			for (k = i + 2;; k += 2) {
-				if (k >= *np || equation[k].level <= level)
-					break;
+		if (!(equation[i].level == level && equation[i].token.operatr == MODULUS))
+			continue;
+		for (k = i + 2;; k += 2) {
+			if (k >= *np || equation[k].level <= level)
+				break;
+		}
+		len1 = k - (i + 1);
+		last_op2 = 0;
+		for (j = loc; j < *np && equation[j].level >= level; j++) {
+			if (equation[j].level == level && equation[j].kind == OPERATOR) {
+				last_op2 = equation[j].token.operatr;
+				continue;
 			}
-			len1 = k - (i + 1);
-			last_op2 = 0;
-			for (j = loc; j < *np && equation[j].level >= level; j++) {
-				if (equation[j].level == level && equation[j].kind == OPERATOR) {
-					last_op2 = equation[j].token.operatr;
-					continue;
+			if (last_op2 == MODULUS) {
+				continue;
+			}
+			last_op2 = MODULUS;
+			op = 0;
+			for (i1 = k = j + 1; k < *np && equation[k].level > level; k += 2) {
+				if (equation[k].level == (level + 1)) {
+					op = equation[k].token.operatr;
+					i1 = k;
 				}
-				if (last_op2 == MODULUS) {
-					continue;
+			}
+			len2 = k - j;
+			switch (op) {
+			case MODULUS:
+				/* simplify (x%n)%n to x%n */
+				len3 = k - (i1 + 1);
+				if (se_compare(&equation[i+1], len1, &equation[i1+1], len3, &diff_sign)) {
+					blt(&equation[i1], &equation[k], (*np - k) * sizeof(token_type));
+					*np -= len3 + 1;
+					return true;
 				}
-				last_op2 = MODULUS;
-				op = 0;
-				for (i1 = k = j + 1; k < *np && equation[k].level > level; k += 2) {
-					if (equation[k].level == (level + 1)) {
-						op = equation[k].token.operatr;
-						i1 = k;
-					}
-				}
-				len2 = k - j;
-				switch (op) {
-				case MODULUS:
-					/* simplify (x%n)%n to x%n */
-					len3 = k - (i1 + 1);
-					if (se_compare(&equation[i+1], len1, &equation[i1+1], len3, &diff_sign)) {
-						blt(&equation[i1], &equation[k], (*np - k) * sizeof(token_type));
-						*np -= len3 + 1;
-						return true;
-					}
+				break;
+			case TIMES:
+				if (!is_integer_expr(&equation[j], len2))
 					break;
-				case TIMES:
-					if (!is_integer_expr(&equation[j], len2))
+				/* simplify (i%n*j)%n to (i*j)%n if j is integer */
+				for (i2 = i1 = j + 1;; i1 += 2) {
+					if (i1 >= k || equation[i1].level == (level + 1)) {
+						for (; i2 < i1; i2 += 2) {
+							if (equation[i2].level == (level + 2)
+							    && equation[i2].token.operatr == MODULUS) {
+								len3 = i1 - (i2 + 1);
+								if (se_compare(&equation[i+1], len1, &equation[i2+1], len3, &diff_sign)) {
+									blt(&equation[i2], &equation[i1], (*np - i1) * sizeof(token_type));
+									*np -= len3 + 1;
+									return true;
+								}
+							}
+						}
+					}
+					if (i1 >= k)
 						break;
-					/* simplify (i%n*j)%n to (i*j)%n if j is integer */
-					for (i2 = i1 = j + 1;; i1 += 2) {
-						if (i1 >= k || equation[i1].level == (level + 1)) {
-							for (; i2 < i1; i2 += 2) {
-								if (equation[i2].level == (level + 2)
-								    && equation[i2].token.operatr == MODULUS) {
+				}
+				break;
+			case PLUS:
+			case MINUS:
+				/* simplify (i%n+j)%n to (i+j)%n */
+				/* and ((i%n)*j+k)%n to (i*j+k)%n, when j is integer */
+				for (i2 = i1 = j + 1, i3 = j - 1;; i1 += 2) {
+					if (i1 >= k || equation[i1].level == (level + 1)) {
+						for (; i2 < i1; i2 += 2) {
+							if (equation[i2].level == (level + 2)) {
+								switch (equation[i2].token.operatr) {
+								case MODULUS:
 									len3 = i1 - (i2 + 1);
 									if (se_compare(&equation[i+1], len1, &equation[i2+1], len3, &diff_sign)) {
 										blt(&equation[i2], &equation[i1], (*np - i1) * sizeof(token_type));
 										*np -= len3 + 1;
 										return true;
 									}
-								}
-							}
-						}
-						if (i1 >= k)
-							break;
-					}
-					break;
-				case PLUS:
-				case MINUS:
-					/* simplify (i%n+j)%n to (i+j)%n */
-					/* and ((i%n)*j+k)%n to (i*j+k)%n, when j is integer */
-					for (i2 = i1 = j + 1, i3 = j - 1;; i1 += 2) {
-						if (i1 >= k || equation[i1].level == (level + 1)) {
-							for (; i2 < i1; i2 += 2) {
-								if (equation[i2].level == (level + 2)) {
-									switch (equation[i2].token.operatr) {
-									case MODULUS:
-										len3 = i1 - (i2 + 1);
-										if (se_compare(&equation[i+1], len1, &equation[i2+1], len3, &diff_sign)) {
-											blt(&equation[i2], &equation[i1], (*np - i1) * sizeof(token_type));
-											*np -= len3 + 1;
-											return true;
-										}
-										break;
-									case TIMES:
-										i2 = i1 - 2;
-										if (!is_integer_expr(&equation[i3+1], i1 - (i3 + 1))) {
-											break;
-										}
-										for (i4 = i3 + 2; i4 < i1; i4 += 2) {
-											if (equation[i4].level == (level + 3)
-											    && equation[i4].token.operatr == MODULUS) {
-												for (i5 = i4 + 2; i5 < i1 && equation[i5].level > (level + 3); i5 += 2)
-													;
-												len3 = i5 - (i4 + 1);
-												if (se_compare(&equation[i+1], len1, &equation[i4+1], len3, &diff_sign)) {
-													blt(&equation[i4], &equation[i5], (*np - i5) * sizeof(token_type));
-													*np -= len3 + 1;
-													return true;
-												}
-											}
-										}
+									break;
+								case TIMES:
+									i2 = i1 - 2;
+									if (!is_integer_expr(&equation[i3+1], i1 - (i3 + 1))) {
 										break;
 									}
+									for (i4 = i3 + 2; i4 < i1; i4 += 2) {
+										if (equation[i4].level == (level + 3)
+										    && equation[i4].token.operatr == MODULUS) {
+											for (i5 = i4 + 2; i5 < i1 && equation[i5].level > (level + 3); i5 += 2)
+												;
+											len3 = i5 - (i4 + 1);
+											if (se_compare(&equation[i+1], len1, &equation[i4+1], len3, &diff_sign)) {
+												blt(&equation[i4], &equation[i5], (*np - i5) * sizeof(token_type));
+												*np -= len3 + 1;
+												return true;
+											}
+										}
+									}
+									break;
 								}
 							}
-							i3 = i1;
 						}
-						if (i1 >= k)
-							break;
+						i3 = i1;
 					}
-					break;
+					if (i1 >= k)
+						break;
 				}
-				/* Remove integer*n multiples in x for x%n by doing */
-				/* polynomial division x/n and replacing with remainder%n. */
-				/* Globals tlhs[] and trhs[] are wiped out by the polynomial division here. */
-				v = 0;
-				if (poly_div(&equation[j], len2, &equation[i+1], len1, &v)) {
-					uf_pplus(tlhs, &n_tlhs);	/* so integer%(integer^integer) isn't simplified to 0 */
-					if (is_integer_expr(tlhs, n_tlhs)) {
-						if ((*np + (n_trhs - len2)) > n_tokens)
-							error_huge();
-						for (k = 0; k < n_trhs; k++)
-							trhs[k].level += level;
-						blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
-						*np += n_trhs - len2;
-						blt(&equation[j], trhs, n_trhs * sizeof(token_type));
-						return true;
-					}
+				break;
+			}
+			/* Remove integer*n multiples in x for x%n by doing */
+			/* polynomial division x/n and replacing with remainder%n. */
+			/* Globals tlhs[] and trhs[] are wiped out by the polynomial division here. */
+			if (poly_div(&equation[j], len2, &equation[i+1], len1, NULL)) {
+				uf_pplus(tlhs, &n_tlhs);	/* so integer%(integer^integer) isn't simplified to 0 */
+				if (is_integer_expr(tlhs, n_tlhs)) {
+					if ((*np + (n_trhs - len2)) > n_tokens)
+						error_huge();
+					for (k = 0; k < n_trhs; k++)
+						trhs[k].level += level;
+					blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
+					*np += n_trhs - len2;
+					blt(&equation[j], trhs, n_trhs * sizeof(token_type));
+					return true;
 				}
 			}
 		}
@@ -979,6 +1034,7 @@ int		*np, loc, level;
 	int	i, j, k;
 	int	last_op2;
 	int	len1, len2;
+	int	rv;
 
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
@@ -992,6 +1048,10 @@ int		*np, loc, level;
 	}
 start:
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
+#if	DEBUG
+		if (equation[i].kind != OPERATOR)
+			error_bug("Bug in poly_gcd_simp().");
+#endif
 		if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
 			for (k = i + 2;; k += 2) {
 				if (k >= *np || equation[k].level <= level)
@@ -1004,8 +1064,14 @@ start:
 					last_op2 = equation[j].token.operatr;
 					continue;
 				}
-				if (last_op2 == DIVIDE) {
+				switch (last_op2) {
+				case DIVIDE:
 					continue;
+				case 0:
+				case TIMES:
+					break;
+				default:
+					error_bug("Expression is corrupt in poly_gcd_simp().");
 				}
 				last_op2 = DIVIDE;
 				for (k = j + 1;; k += 2) {
@@ -1013,7 +1079,7 @@ start:
 						break;
 				}
 				len2 = k - j;
-				if (poly2_gcd(&equation[i+1], len1, &equation[j], len2, 0L)) {
+				if ((rv = poly2_gcd(&equation[i+1], len1, &equation[j], len2, 0L, true)) > 0) {
 store_code:
 					for (k = 0; k < n_tlhs; k++)
 						tlhs[k].level += level;
@@ -1034,7 +1100,7 @@ store_code:
 					modified = true;
 					goto start;
 				}
-				if (poly2_gcd(&equation[j], len2, &equation[i+1], len1, 0L)) {
+				if (rv == 0 && poly2_gcd(&equation[j], len2, &equation[i+1], len1, 0L, true) > 0) {
 					k = j - 1;
 					j = i + 1;
 					i = k;
@@ -1052,6 +1118,12 @@ store_code:
 /*
  * This routine is a division simplifier for equation sides.
  * Check for divides and do polynomial and smart division.
+ * For example: (c - b + a)/(b - a) will be converted to (c/(b - a)) - 1.
+ * Will only make expressions with algebraic fractions smaller,
+ * otherwise it would go into an infinite loop.
+ * An expression with less operators is judged to be smaller.
+ * Complex fractions (fractions within fractions) are sometimes
+ * created by this, especially if quick_flag is false.
  *
  * Return true if expression was simplified.
  */
@@ -1059,16 +1131,18 @@ int
 div_remainder(equation, np, poly_flag, quick_flag)
 token_type	*equation;
 int		*np;
-int		poly_flag;
-int		quick_flag;
+int		poly_flag;	/* if true, try polynomial division first, then smart division */
+int		quick_flag;	/* if true, keep algebraic fractions simpler */
 {
 	int	rv = false;
 
+	debug_string(3, "Entering div_remainder().");
 	if (quick_flag)
 		group_proc(equation, np);
 	rv = pdiv_recurse(equation, np, 0, 1, poly_flag);
 	if (quick_flag)
 		organize(equation, np);
+	debug_string(3, "Leaving div_remainder().");
 	return rv;
 }
 
@@ -1081,155 +1155,153 @@ int		*np, loc, level, code;
 	int	i, j, k;
 	int	op, op2, last_op2;
 	int	len1, len2, real_len1;
-	long	v;
 	int	rv = 0;
 	int	flag, power_flag, zero_remainder;
 
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-		if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
-			for (k = i + 2;; k += 2) {
-				if (k >= *np || equation[k].level <= level)
-					break;
+		if (!(equation[i].level == level && equation[i].token.operatr == DIVIDE))
+			continue;
+		for (k = i + 2;; k += 2) {
+			if (k >= *np || equation[k].level <= level)
+				break;
+		}
+		len1 = real_len1 = k - (i + 1);
+		last_op2 = 0;
+		for (j = loc; j < *np && equation[j].level >= level; j++) {
+			if (equation[j].level == level && equation[j].kind == OPERATOR) {
+				last_op2 = equation[j].token.operatr;
+				continue;
 			}
-			len1 = real_len1 = k - (i + 1);
-			last_op2 = 0;
-			for (j = loc; j < *np && equation[j].level >= level; j++) {
-				if (equation[j].level == level && equation[j].kind == OPERATOR) {
-					last_op2 = equation[j].token.operatr;
-					continue;
+			if (last_op2 == DIVIDE) {
+				continue;
+			}
+			last_op2 = DIVIDE;
+			op = 0;
+			for (k = j + 1; k < *np && equation[k].level > level; k += 2) {
+				if (equation[k].level == (level + 1)) {
+					op = equation[k].token.operatr;
 				}
-				if (last_op2 == DIVIDE) {
-					continue;
-				}
-				last_op2 = DIVIDE;
-				op = 0;
-				for (k = j + 1; k < *np && equation[k].level > level; k += 2) {
-					if (equation[k].level == (level + 1)) {
-						op = equation[k].token.operatr;
+			}
+			if (op != PLUS && op != MINUS) {
+				continue;
+			}
+			len2 = k - j;
+			flag = code;
+			power_flag = false;
+			op = 0;
+			op2 = 0;
+			for (k = i + 2; k < *np && equation[k].level > level; k += 2) {
+				if (equation[k].level == level + 3) {
+					switch (equation[k].token.operatr) {
+					case PLUS:
+					case MINUS:
+						op2 = PLUS;
 					}
-				}
-				if (op != PLUS && op != MINUS) {
-					continue;
-				}
-				len2 = k - j;
-				flag = code;
-				power_flag = false;
-				op = 0;
-				op2 = 0;
-				for (k = i + 2; k < *np && equation[k].level > level; k += 2) {
-					if (equation[k].level == level + 3) {
-						switch (equation[k].token.operatr) {
-						case PLUS:
-						case MINUS:
-							op2 = PLUS;
-						}
-					} else if (equation[k].level == level + 2) {
-						op = equation[k].token.operatr;
-					} else if (equation[k].level == level + 1) {
-						if (equation[k].token.operatr == POWER
-						    && (op == PLUS || op == MINUS || (op == TIMES && op2 == PLUS))) {
-							power_flag = true;
-							len1 = k - (i + 1);
-						}
-						break;
+				} else if (equation[k].level == level + 2) {
+					op = equation[k].token.operatr;
+				} else if (equation[k].level == level + 1) {
+					if (equation[k].token.operatr == POWER
+					    && (op == PLUS || op == MINUS || (op == TIMES && op2 == PLUS))) {
+						power_flag = true;
+						len1 = k - (i + 1);
 					}
+					break;
 				}
+			}
 next_thingy:
-				if (!power_flag) {
-					len1 = real_len1;
-				}
-				v = 0;
-				if (flag || power_flag) {
-					rv = poly_div(&equation[j], len2, &equation[i+1], len1, &v);
-				} else {
-					rv = smart_div(&equation[j], len2, &equation[i+1], len1);
-				}
-				zero_remainder = (rv > 0 && REMAINDER_IS_ZERO());
-				if (power_flag && !zero_remainder) {
-					rv = 0;
-				}
-				if (rv > 0) {
-					if ((n_tlhs + 2 + n_trhs + len1) > n_tokens)
-						error_huge();
-					for (k = 0; k < n_tlhs; k++)
-						tlhs[k].level++;
-					tlhs[n_tlhs].level = 1;
-					tlhs[n_tlhs].kind = OPERATOR;
-					tlhs[n_tlhs].token.operatr = PLUS;
-					n_tlhs++;
-					for (k = 0; k < n_trhs; k++)
-						trhs[k].level += 2;
-					blt(&tlhs[n_tlhs], trhs, n_trhs * sizeof(token_type));
-					n_tlhs += n_trhs;
-					tlhs[n_tlhs].level = 2;
-					tlhs[n_tlhs].kind = OPERATOR;
-					tlhs[n_tlhs].token.operatr = DIVIDE;
-					n_tlhs++;
-					k = n_tlhs;
-					blt(&tlhs[n_tlhs], &equation[i+1], len1 * sizeof(token_type));
-					n_tlhs += len1;
-					for (; k < n_tlhs; k++)
-						tlhs[k].level += 2;
-					side_debug(3, &equation[j], len2);
-					side_debug(3, &equation[i+1], len1);
-					simpb_side(tlhs, &n_tlhs, false, true, 3);	/* parameters should match what's used in simpa_side() */
-					side_debug(3, tlhs, n_tlhs);
-					if (power_flag) {
-						k = (var_count(tlhs, n_tlhs) <= var_count(&equation[j], len2));
-					} else {
-						k = ((var_count(tlhs, n_tlhs) + (n_tlhs >= len1 + 1 + len2)) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
-					}
-					if (k) {
-						for (k = 0; k < n_tlhs; k++)
-							tlhs[k].level += level;
-						if (power_flag) {
-							if ((*np - len2 + n_tlhs + 2) > n_tokens)
-								error_huge();
-							for (k = i + 2 + len1; k <= i + real_len1; k++) {
-								equation[k].level++;
-							}
-							blt(&equation[i+real_len1+3], &equation[k], (*np - k) * sizeof(token_type));
-							*np += 2;
-							equation[k].level = level + 2;
-							equation[k].kind = OPERATOR;
-							equation[k].token.operatr = MINUS;
-							k++;
-							equation[k].level = level + 2;
-							equation[k].kind = CONSTANT;
-							equation[k].token.constant = 1.0;
-							if (i < j) {
-								j += 2;
-							}
-						} else {
-							if ((*np - (len1 + 1 + len2) + n_tlhs) > n_tokens)
-								error_huge();
-							blt(&equation[i], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(token_type));
-							*np -= len1 + 1;
-							if (i < j) {
-								j -= len1 + 1;
-							}
-						}
-						blt(&equation[j+n_tlhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
-						*np -= len2 - n_tlhs;
-						blt(&equation[j], tlhs, n_tlhs * sizeof(token_type));
-						if (flag || power_flag) {
-							debug_string(1, _("Polynomial division successful."));
-						} else {
-							debug_string(1, _("Smart division successful."));
-						}
-						side_debug(3, equation, *np);
-						return true;
-					}
-				}
+			if (!power_flag) {
+				len1 = real_len1;
+			}
+			if (flag || power_flag) {
+				rv = poly_div(&equation[j], len2, &equation[i+1], len1, NULL);
+			} else {
+				rv = smart_div(&equation[j], len2, &equation[i+1], len1);
+			}
+			zero_remainder = (rv > 0 && REMAINDER_IS_ZERO());
+			if (power_flag && !zero_remainder) {
+				rv = 0;
+			}
+			if (rv > 0) { /* if successful and the result is smaller than the original: */
+				if ((n_tlhs + 2 + n_trhs + len1) > n_tokens)
+					error_huge();
+				for (k = 0; k < n_tlhs; k++)
+					tlhs[k].level++;
+				tlhs[n_tlhs].level = 1;
+				tlhs[n_tlhs].kind = OPERATOR;
+				tlhs[n_tlhs].token.operatr = PLUS;
+				n_tlhs++;
+				for (k = 0; k < n_trhs; k++)
+					trhs[k].level += 2;
+				blt(&tlhs[n_tlhs], trhs, n_trhs * sizeof(token_type));
+				n_tlhs += n_trhs;
+				tlhs[n_tlhs].level = 2;
+				tlhs[n_tlhs].kind = OPERATOR;
+				tlhs[n_tlhs].token.operatr = DIVIDE;
+				n_tlhs++;
+				k = n_tlhs;
+				blt(&tlhs[n_tlhs], &equation[i+1], len1 * sizeof(token_type));
+				n_tlhs += len1;
+				for (; k < n_tlhs; k++)
+					tlhs[k].level += 2;
+				side_debug(3, &equation[j], len2);
+				side_debug(3, &equation[i+1], len1);
+				simpb_side(tlhs, &n_tlhs, false, true, 3);	/* parameters should match what's used in simpa_side() */
+				side_debug(3, tlhs, n_tlhs);
 				if (power_flag) {
-					power_flag = false;
-					goto next_thingy;
+					k = (var_count(tlhs, n_tlhs) <= var_count(&equation[j], len2));
+				} else {
+					k = ((var_count(tlhs, n_tlhs) + (n_tlhs >= len1 + 1 + len2)) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
 				}
-				if (flag == code) {
-					flag = !flag;
-					goto next_thingy;
+				if (k) {
+					for (k = 0; k < n_tlhs; k++)
+						tlhs[k].level += level;
+					if (power_flag) {
+						if ((*np - len2 + n_tlhs + 2) > n_tokens)
+							error_huge();
+						for (k = i + 2 + len1; k <= i + real_len1; k++) {
+							equation[k].level++;
+						}
+						blt(&equation[i+real_len1+3], &equation[k], (*np - k) * sizeof(token_type));
+						*np += 2;
+						equation[k].level = level + 2;
+						equation[k].kind = OPERATOR;
+						equation[k].token.operatr = MINUS;
+						k++;
+						equation[k].level = level + 2;
+						equation[k].kind = CONSTANT;
+						equation[k].token.constant = 1.0;
+						if (i < j) {
+							j += 2;
+						}
+					} else {
+						if ((*np - (len1 + 1 + len2) + n_tlhs) > n_tokens)
+							error_huge();
+						blt(&equation[i], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(token_type));
+						*np -= len1 + 1;
+						if (i < j) {
+							j -= len1 + 1;
+						}
+					}
+					blt(&equation[j+n_tlhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
+					*np -= len2 - n_tlhs;
+					blt(&equation[j], tlhs, n_tlhs * sizeof(token_type));
+					if (flag || power_flag) {
+						debug_string(1, _("Polynomial division successful."));
+					} else {
+						debug_string(1, _("Smart division successful."));
+					}
+					side_debug(3, equation, *np);
+					return true;
 				}
 			}
+			if (power_flag) {
+				power_flag = false;
+				goto next_thingy;
+			}
+			if (flag == code) {
+				flag = !flag;
+				goto next_thingy;
+			}
 		}
 	}
 	for (i = loc; i < *np && equation[i].level >= level;) {
@@ -1246,9 +1318,13 @@ next_thingy:
 }
 
 /*
- * Do polynomial division.
+ * Do generalized and accurate polynomial division.
+ * This is the main polynomial division function for Mathomatic.
+ * Works with any number of variables and any pair of expressions.
  *
- * Returns non-zero if successful, 2 if remainder is 0.
+ * Returns non-zero if successful: 2 if remainder is 0,
+ * 1 if result is smaller than the original two expressions,
+ * negative if result is larger.
  * Quotient is returned in tlhs[] and remainder in trhs[].
  *
  * If *vp is 0, automatically select the best base variable and return it in *vp.
@@ -1259,7 +1335,7 @@ token_type	*d1;		/* pointer to dividend */
 int		len1;		/* length of dividend */
 token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
-long		*vp;		/* variable pointer to base variable */
+long		*vp;		/* pointer to base variable */
 {
 	int		i;
 	int		rv;
@@ -1284,7 +1360,8 @@ long		*vp;		/* variable pointer to base variable */
 }
 
 /*
- * Do the actual polynomial division using a simple generalized polynomial long division algorithm.
+ * Do the actual polynomial division using a simple,
+ * generalized, polynomial long division algorithm.
  */
 static int
 poly_div_sub(d1, len1, d2, len2, vp)
@@ -1302,18 +1379,21 @@ long		*vp;
 	double		last_power, divisor_power, d;
 	int		sum_size;
 	int		last_count, count;	/* dividend number of terms raised to the highest power */
+	long		tmp_v = 0;
 
+	if (vp == NULL)
+		vp = &tmp_v;
 	if (len1 > n_tokens || len2 > n_tokens)
 		return false;
 	/* Copy the source polynomials to where we want them (tlhs and trhs). */
 	if (trhs != d1) {
 		blt(trhs, d1, len1 * sizeof(token_type));
-		n_trhs = len1;
 	}
+	n_trhs = len1;
 	if (tlhs != d2) {
 		blt(tlhs, d2, len2 * sizeof(token_type));
-		n_tlhs = len2;
 	}
+	n_tlhs = len2;
 	/* Do the basic unfactoring and simplification. */
 	uf_simp(trhs, &n_trhs);
 	uf_simp(tlhs, &n_tlhs);
@@ -1442,7 +1522,7 @@ long		*vp;
 				else
 					return -1;
 			}
-			return true;
+			return 1;
 		} else if (d < last_power) {
 			last_power = d;
 			last_count = count;
@@ -1707,7 +1787,7 @@ end_div2:
 }
 
 /*
- * Return the size of a subexpression,
+ * Return the size of a sub-expression,
  * minus any constant multiplier.
  */
 int
