@@ -16,13 +16,15 @@ void set_sign_array(void);
 int next_sign(long *vp);
 void clear_all(void);
 int alloc_espace(int i);
+int alloc_to_espace(int en);
 int alloc_next_espace(void);
 int next_espace(void);
 void copy_espace(int src, int dest);
 int solved_equation(int i);
 int found_var(token_type *p1, int n, long v);
 int var_in_equation(int i, long v);
-void subst_var_with_exp(token_type *equation, int *np, token_type *expression, int len, long v);
+void rename_var_in_es(int en, long from_v, long to_v);
+int subst_var_with_exp(token_type *equation, int *np, token_type *expression, int len, long v);
 int min_level(token_type *expression, int n);
 int get_default_en(char *cp);
 int get_expr(token_type *equation, int *np);
@@ -53,10 +55,12 @@ int check_divide_by_zero(double denominator);
 /* cmds.c */
 int plot_cmd(char *cp);
 int version_cmd(char *cp);
+long max_memory_usage(void);
 int version_report(void);
 int solve_cmd(char *cp);
 int sum_cmd(char *cp);
 int product_cmd(char *cp);
+int for_cmd(char *cp);
 int optimize_cmd(char *cp);
 int push_cmd(char *cp);
 int push_en(int en);
@@ -91,6 +95,7 @@ int unfactor_cmd(char *cp);
 int fraction_cmd(char *cp);
 int quit_cmd(char *cp);
 int read_cmd(char *cp);
+int read_file(char *cp);
 int read_sub(FILE *fp);
 int edit_cmd(char *cp);
 int save_cmd(char *cp);
@@ -137,6 +142,8 @@ double gcd_verified(double d1, double d2);
 double my_round(double d1);
 int f_to_fraction(double d, double *numeratorp, double *denominatorp);
 int make_fractions(token_type *equation, int *np);
+int make_simple_fractions(token_type *equation, int *np);
+int make_mixed_fractions(token_type *equation, int *np);
 /* globals.c */
 /* help.c */
 int parse(int n, char *cp);
@@ -147,10 +154,11 @@ int shell_out(char *cp);
 char *parse_var2(long *vp, char *cp);
 int display_command(int i);
 int read_examples(char **cpp);
+void underline_title(int count);
 int help_cmd(char *cp);
 /* integrate.c */
 void make_powers(token_type *equation, int *np, long v);
-int int_dispatch(token_type *equation, int *np, long v, int (*func)(void));
+int int_dispatch(token_type *equation, int *np, long v, int (*func)(token_type *equation, int *np, int loc, int eloc, long v));
 int integrate_cmd(char *cp);
 int laplace_cmd(char *cp);
 int nintegrate_cmd(char *cp);
@@ -161,6 +169,7 @@ void default_color(void);
 int list1_sub(int n, int export_flag);
 int list_sub(int n);
 void list_debug(int level, token_type *p1, int n1, token_type *p2, int n2);
+char *var_name(long v);
 int list_var(long v, int lang_code);
 int list_proc(token_type *p1, int n, int export_flag);
 char *list_equation(int n, int export_flag);
@@ -168,11 +177,12 @@ char *list_expression(token_type *p1, int n, int export_flag);
 int list_string(token_type *p1, int n, char *string, int export_flag);
 int list_string_sub(token_type *p1, int n, int outflag, char *string, int export_flag);
 int int_expr(token_type *p1, int n);
-int list_c_equation(int en, enum language_list language, int int_flag);
+int list_code_equation(int en, enum language_list language, int int_flag);
 int list_code(token_type *equation, int *np, enum language_list language, int int_flag);
 int flist_equation(int n);
 /* main.c */
-void usage(void);
+void display_startup_message(FILE *fp);
+void usage(FILE *fp);
 int main(int argc, char **argv);
 void main_io_loop(void);
 int set_signals(void);
@@ -180,18 +190,20 @@ int load_rc(void);
 void fphandler(int sig);
 void inthandler(int sig);
 void alarmhandler(int sig);
+void exithandler(int sig);
 void resizehandler(int sig);
 void exit_program(int exit_value);
 /* parse.c */
 void str_tolower(char *cp);
 void put_up_arrow(int cnt, char *cp);
 int isvarchar(int ch);
+int is_mathomatic_operator(int ch);
 void binary_parenthesize(token_type *p1, int n, int i);
 void handle_negate(token_type *equation, int *np);
 void give_priority(token_type *equation, int *np);
 char *parse_section(token_type *equation, int *np, char *cp, int allow_space);
 char *parse_equation(int n, char *cp);
-char *parse_expr(token_type *equation, int *np, char *cp);
+char *parse_expr(token_type *equation, int *np, char *cp, int allow_space);
 char *parse_var(long *vp, char *cp);
 void remove_trailing_spaces(char *cp);
 void set_error_level(char *cp);
@@ -199,11 +211,13 @@ int var_is_const(long v, double *dp);
 int subst_constants(token_type *equation, int *np);
 int my_strlcpy(char *dest, char *src, int n);
 /* poly.c */
+int poly_in_v_sub(token_type *p1, int n, long v, int allow_divides);
 int poly_in_v(token_type *p1, int n, long v, int allow_divides);
 int poly_factor(token_type *equation, int *np, int do_repeat);
 int remove_factors(void);
 int poly_gcd(token_type *larger, int llen, token_type *smaller, int slen, long v);
-int poly2_gcd(token_type *larger, int llen, token_type *smaller, int slen, long v);
+int poly2_gcd(token_type *larger, int llen, token_type *smaller, int slen, long v, int require_additive);
+int is_integer_var(long v);
 int is_integer_expr(token_type *p1, int n);
 int mod_simp(token_type *equation, int *np);
 int poly_gcd_simp(token_type *equation, int *np);
@@ -218,12 +232,15 @@ int find_greatest_power(token_type *p1, int n1, long *vp1, double *pp1, int *tp1
 void organize(token_type *equation, int *np);
 void elim_loop(token_type *equation, int *np);
 void simp_ssub(token_type *equation, int *np, long v, double d, int power_flag, int times_flag, int fc_level);
-void simp_sub(int n);
-void simp_side(token_type *equation, int *np);
+void simp_equation(int n);
+void mid_simp_side(token_type *equation, int *np);
+void mid_simp_equation(int n);
 void simps_side(token_type *equation, int *np, int zsolve);
 void simpv_side(token_type *equation, int *np, long v);
+void simpv_equation(int n, long v);
 void factorv(token_type *equation, int *np, long v);
 void calc_simp(token_type *equation, int *np);
+void approximate(token_type *equation, int *np);
 void simp_i(token_type *equation, int *np);
 void simp_divide(token_type *equation, int *np);
 void simp2_divide(token_type *equation, int *np, long v, int fc_level);
@@ -231,11 +248,13 @@ void simpb_side(token_type *equation, int *np, int uf_power_flag, int power_flag
 void simple_frac_side(token_type *equation, int *np);
 void simpa_side(token_type *equation, int *np, int quick_flag, int frac_flag);
 void simpa_repeat_side(token_type *equation, int *np, int quick_flag, int frac_flag);
+void simple_frac_repeat_side(token_type *equation, int *np);
 int simp_loop(token_type *equation, int *np);
 int simp_pp(token_type *equation, int *np);
 int integer_root_simp(token_type *equation, int *np);
 int simp_constant_power(token_type *equation, int *np);
 int simp2_power(token_type *equation, int *np);
+double fixed_fmod(double k1, double k2);
 int combine_constants(token_type *equation, int *np, int iflag);
 int calc(int *op1p, double *k1p, int op2, double k2);
 int elim_k(token_type *equation, int *np);
