@@ -1,7 +1,7 @@
 /*
  * Mathomatic simplifying routines.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -30,10 +30,10 @@
  */
 #define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 6)
 
-static int	org_recurse();
-static int	const_recurse();
-static int	compare_recurse();
-static int	order_recurse();
+static int org_recurse(token_type *equation, int *np, int loc, int level, int *elocp);
+static int const_recurse(token_type *equation, int *np, int loc, int level, int iflag);
+static int compare_recurse(token_type *p1, int n1, int l1, token_type *p2, int n2, int l2, int *diff_signp);
+static int order_recurse(token_type *equation, int *np, int loc, int level);
 
 /*
  * Fix up levels of parentheses in an equation side.
@@ -46,7 +46,7 @@ organize(equation, np)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
-	if (*np <= 0 || (*np & 1) != 1) {
+	if (*np <= 0 || (*np & 1) == 0) {
 		printf("Bad expression size = %d.\n", *np);
 		error_bug("Internal error: organize() called with bad expression size.");
 	}
@@ -56,7 +56,7 @@ int		*np;		/* pointer to length of equation side */
 	org_recurse(equation, np, 0, 1, NULL);
 }
 
-static void
+static inline void
 org_up_level(bp, ep, level, invert)
 token_type	*bp, *ep;
 int		level, invert;
@@ -172,7 +172,18 @@ int		*np;		/* pointer to length of equation side */
 	if (abort_flag) {
 		/* Control-C pressed, gracefully return to main prompt and leave unsimplified */
 		abort_flag = false;
+#if	DEBUG && !SILENT
+		char	*cp, buf[100];
+
+		my_strlcpy(prompt_str, _("Enter debug level or empty line to abort: "), sizeof(prompt_str));
+		if ((cp = get_string(buf, sizeof(buf))) == NULL || *cp == '\0') {
+			longjmp(jmp_save, 13);
+		} else {
+			debug_level = decstrtol(cp, NULL);
+		}
+#else
 		longjmp(jmp_save, 13);
+#endif
 	}
 	side_debug(6, equation, *np);
 	do {
@@ -233,7 +244,7 @@ int	n;	/* equation space number to simplify */
 	if (n_lhs[n] <= 0)
 		return;
 	simp_loop(lhs[n], &n_lhs[n]);
-	if (n_rhs[n]) {
+	if (n_rhs[n] > 0) {
 		simp_loop(rhs[n], &n_rhs[n]);
 	}
 }
@@ -243,11 +254,27 @@ int	n;	/* equation space number to simplify */
  * Trivial factoring is done.
  */
 void
-simp_side(equation, np)
+mid_simp_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
 {
-	simp_ssub(equation, np, 0L, 1.0, true, true, 6 /* was 2 */);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 6);
+}
+
+/*
+ * For quick, mid-range simplification of an equation space.
+ * Trivial factoring is done.
+ */
+void
+mid_simp_equation(n)
+int	n;	/* equation space number to simplify */
+{
+	if (n_lhs[n] <= 0)
+		return;
+	mid_simp_side(lhs[n], &n_lhs[n]);
+	if (n_rhs[n] > 0) {
+		mid_simp_side(rhs[n], &n_rhs[n]);
+	}
 }
 
 /*
@@ -262,7 +289,7 @@ int		zsolve;		/* true for solving for zero */
 	elim_loop(equation, np);
 	simp_constant_power(equation, np);
 	do {
-		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 6 /* was 2 */);
+		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 6);
 	} while (super_factor(equation, np, 0));
 }
 
@@ -275,20 +302,36 @@ token_type	*equation;	/* pointer to the beginning of equation side to factor */
 int		*np;		/* pointer to length of equation side */
 long		v;		/* variable to factor, 0 for all variables */
 {
-	if (*np == 0)
-		return;
-	simp_ssub(equation, np, v, 0.0, v == 0, true, 6 /* was 2 */);
+	simp_ssub(equation, np, v, 0.0, v == 0, true, 6);
 }
 
+/*
+ * For factoring an equation space like the factor command.
+ * Trivial factoring is done.
+ */
+void
+simpv_equation(n, v)
+int	n;	/* equation space number to simplify */
+long	v;	/* Mathomatic variable to factor or 0 */
+{
+	if (n_lhs[n] <= 0)
+		return;
+	simpv_side(lhs[n], &n_lhs[n], v);
+	if (n_rhs[n] > 0) {
+		simpv_side(rhs[n], &n_rhs[n], v);
+	}
+}
+
+
 /*
  * Factor only the specified variable "v".
- * If v is IMAGINARY, do complex number simplification.
+ * If v is IMAGINARY, also do complex number approximation and simplification.
  */
 void
 factorv(equation, np, v)
 token_type	*equation;	/* pointer to the beginning of equation side to factor */
 int		*np;		/* pointer to equation side length */
-long		v;		/* variable to factor */
+long		v;		/* Mathomatic variable to factor */
 {
 	if (v == IMAGINARY) {
 		approximate_complex_roots(equation, np);
@@ -311,19 +354,34 @@ int		*np;
 {
 	approximate_roots = true;
 	subst_constants(equation, np);
-	simp_side(equation, np);
+	mid_simp_side(equation, np);
 	uf_power(equation, np);
 	factorv(equation, np, IMAGINARY);
 	ufactor(equation, np);
 	factorv(equation, np, IMAGINARY);
 	uf_simp(equation, np);
 	factorv(equation, np, IMAGINARY);
-	simp_side(equation, np);
-	make_fractions(equation, np);
+	mid_simp_side(equation, np);
+	make_simple_fractions(equation, np);
 	uf_tsimp(equation, np);
 	approximate_roots = false;
 }
 
+/*
+ * Approximate an equation side for the approximate command.
+ */
+void
+approximate(equation, np)
+token_type	*equation;
+int		*np;
+{
+	subst_constants(equation, np);
+	approximate_roots = true;
+	simp_loop(equation, np);
+	factorv(equation, np, IMAGINARY);
+	approximate_roots = false;
+}
+
 /*
  * Try to eliminate imaginary numbers (i#) from an equation side by converting
  * expressions like (i#*(b^.5)) to ((-1*b)^.5).
@@ -490,7 +548,7 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 				simp2_divide(equation, np, va[i].v, fc_level);
 		}
 		simp2_divide(equation, np, 0L, fc_level);
-		/* factor all subexpressions in order of most frequently occurring variables */
+		/* factor all sub-expressions in order of most frequently occurring variables */
 		for (i = 0; i < vc; i++) {
 			while (factor_plus(equation, np, va[i].v, 0.0)) {
 				simp2_divide(equation, np, 0L, fc_level);
@@ -502,6 +560,13 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 		}
 	}
 	simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
+#if	0	/* Seems to complicate and change constants slightly when enabled. */
+	if (power_flag) {
+		make_simple_fractions(equation, np);
+		factor_power(equation, np);
+		simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
+	}
+#endif
 }
 
 /*
@@ -517,25 +582,31 @@ int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0)
 		return;
-
 	do {
 		do {
 			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 5);
+				simp_ssub(equation, np, 0L, 0.0, false, true, 5);
 			} while (poly_gcd_simp(equation, np));
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
 	side_debug(2, equation, *np);
 
-	make_fractions(equation, np);
+	make_simple_fractions(equation, np);
 	uf_tsimp(equation, np);
+#if	0	/* Not really needed, but makes it end up the same as the simplify command. */
+	make_simple_fractions(equation, np);
+	uf_power(equation, np);
+	integer_root_simp(equation, np);
+	simpb_side(equation, np, true, true, 3);
+#endif
 	poly_factor(equation, np, true);
 	simpb_side(equation, np, true, false, 2);
+	simpb_side(equation, np, true, false, 2);	/* Added for thoroughness, making sure everything is uf_power()ed. */
 	fractions_and_group(equation, np);
 }
 
 /*
- * This is the slow and thorough simplify.
+ * This is the slow and thorough simplify of the simplify command.
  * Applies many equivalent algebraic transformations and their inverses (like unfactor and factor),
  * then does generalized polynomial simplifications.
  *
@@ -554,8 +625,9 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 
 	if (*np <= 1)	/* no need to simplify a single constant or variable */
 		return;
+	debug_string(2, "Simplify input:");
 	side_debug(2, equation, *np);
-	simpb_side(equation, np, false, true, 1);
+	simpb_side(equation, np, false, true, 5);
 	if (rationalize_denominators) {
 		rationalize(equation, np);
 	}
@@ -563,7 +635,7 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	uf_times(equation, np);
 	simp_loop(equation, np);
 
-/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
+/* Here is the only place in Mathomatic that we do complete modulus (%) simplification: */
 	uf_pplus(equation, np);
 	uf_repeat(equation, np);
 	do {
@@ -574,31 +646,37 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	simp_i(equation, np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 5);
 	unsimp_power(equation, np);
 	uf_neg_help(equation, np);
 	uf_times(equation, np);
+	simp_loop(equation, np);
 	do {
 		do {
-			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 2);
-			} while (poly_gcd_simp(equation, np));
+			simp_ssub(equation, np, 0L, 1.0, false, true, 6 /* was 2 */);
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
+	if (poly_gcd_simp(equation, np)) {
+		simp_ssub(equation, np, 0L, 1.0, false, true, 6 /* was 2 */);
+	}
+	side_debug(2, equation, *np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
 	factorv(equation, np, IMAGINARY);
 	uf_pplus(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, false, 1);
+	simp_ssub(equation, np, 0L, 1.0, true, false, 5);
+	if (poly_gcd_simp(equation, np)) {
+		factorv(equation, np, IMAGINARY);
+		uf_pplus(equation, np);
+		simp_ssub(equation, np, 0L, 1.0, true, false, 5);
+	}
 	uf_times(equation, np);
 	uf_pplus(equation, np);
 	factorv(equation, np, IMAGINARY);
 	uf_power(equation, np);
 	do {
 		do {
-			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 6 /* was 2 */);
-			} while (poly_gcd_simp(equation, np));
+			simp_ssub(equation, np, 0L, 1.0, false, true, 6);
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
 
@@ -615,7 +693,11 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 		/* an error occurred, restore the original expression */
 		*np = n_tlhs;
 		blt(equation, tlhs, n_tlhs * sizeof(token_type));
-		debug_string(1, "Simplify not expanding fully, due to oversized expression.");
+		if (i == 14) {
+			debug_string(1, "Simplify not expanding fully, due to oversized expression.");
+		} else {
+			debug_string(0, "Simplify not expanding fully, due to some error.");
+		}
 		partial_flag = true;	/* expand less */
 		uf_tsimp(equation, np);
 	} else {
@@ -657,12 +739,12 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	}
 /*	simpb_side(equation, np, true, true, 1); */
 	simp_constant_power(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 5 /* was 1 */);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 5);
 	unsimp_power(equation, np);
-	make_fractions(equation, np);
+	make_simple_fractions(equation, np);
 	factor_power(equation, np);
 	uf_tsimp(equation, np);
-	make_fractions(equation, np);
+	make_simple_fractions(equation, np);
 	uf_power(equation, np);
 	integer_root_simp(equation, np);
 	simpb_side(equation, np, true, true, 3);
@@ -685,6 +767,8 @@ int		*np;		/* pointer to length of the equation side */
 int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
 int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
 {
+	if (*np <= 1)	/* no need to simplify a single constant or variable */
+		return;
 	simpa_side(equation, np, quick_flag, frac_flag);
 	if (repeat_flag) {
 		do {
@@ -699,6 +783,34 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	}
 }
 
+/*
+ * This routine is used by the fraction command.
+ * It repeats the simplification until the smallest expression is achieved,
+ * if repeat_flag is true.
+ *
+ * Globals tes[], tlhs[], and trhs[] are wiped out.
+ */
+void
+simple_frac_repeat_side(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of the equation side */
+{
+	if (*np <= 0)
+		return;
+	simple_frac_side(equation, np);
+	if (repeat_flag) {
+		do {
+			n_tes = *np;
+			blt(tes, equation, n_tes * sizeof(token_type));
+			simple_frac_side(equation, np);
+		} while (*np < n_tes);
+		if (*np != n_tes) {
+			*np = n_tes;
+			blt(equation, tes, n_tes * sizeof(token_type));
+		}
+	}
+}
+
 /*
  * Commonly used quick simplify routine that doesn't factor.
  *
@@ -795,9 +907,9 @@ int		*np;
 }
 
 /*
- * Simplify things like 12^(1/2) to 2*3^(1/2).
+ * Simplify surds like 12^(1/2) to 2*3^(1/2).
  *
- * Return true if expression was modified.
+ * Return true if equation side was modified.
  */
 int
 integer_root_simp(equation, np)
@@ -807,41 +919,42 @@ int		*np;
 	int	modified = false;
 	int	i, j, k;
 	int	level;
-	double	d1, d2;
-	int	root;
+	double	d1, d2, numerator, denominator;
 
-	for (i = 1; i < (*np - 2); i += 2) {
+	for (i = 1; (i + 3) < *np; i += 2) {
+		if (equation[i].kind != OPERATOR) {
+			error_bug("Bug in integer_root_simp().");
+		}
 		if (equation[i].token.operatr == POWER) {
 			level = equation[i].level;
 			if (equation[i-1].level == level
+			    && equation[i+1].level == level + 1
 			    && equation[i+2].level == level + 1
-			    && equation[i+2].token.operatr == DIVIDE
 			    && equation[i+3].level == level + 1
+			    && equation[i+2].token.operatr == DIVIDE
 			    && equation[i-1].kind == CONSTANT
 			    && equation[i+1].kind == CONSTANT
 			    && equation[i+3].kind == CONSTANT) {
-				if (i + 4 < *np && equation[i+4].level >= level)
+				if ((i + 4) < *np && equation[i+4].level >= level)
 					continue;
-				d2 = equation[i+1].token.constant;
-				if (d2 < 1.0 || d2 > 50.0 || fmod(d2, 1.0) != 0.0)
+				numerator = equation[i+1].token.constant;
+				if (numerator > 50.0 || numerator < 1.0 || fmod(numerator, 1.0) != 0.0)
 					continue;
 				errno = 0;
-				d2 = pow(equation[i-1].token.constant, d2);
+				d2 = pow(equation[i-1].token.constant, numerator);
 				if (errno) {
 					continue;
 				}
-				if (equation[i+3].token.constant > 50.0 || equation[i+3].token.constant < 2.0)
-					continue;
-				root = equation[i+3].token.constant;
-				if (root != equation[i+3].token.constant || root < 2)
+				denominator = equation[i+3].token.constant;
+				if (denominator > 50.0 || denominator < 2.0 || fmod(denominator, 1.0) != 0.0)
 					continue;
 				if (!factor_one(d2))
 					continue;
 				d1 = 1.0;
 				for (j = 0; j < uno; j++) {
-					while (ucnt[j] >= root) {
+					while (ucnt[j] >= denominator) {
 						d1 *= unique[j];
-						ucnt[j] -= root;
+						ucnt[j] -= denominator;
 					}
 				}
 				if (d1 == 1.0)
@@ -851,21 +964,19 @@ int		*np;
 				}
 				equation[i+1].token.constant = 1.0;
 				equation[i-1].token.constant = multiply_out_unique();
-				k = i - 1;
-				j = i + 4;
-				for (; k < j; k++) {
+				for (k = i - 1; k < i + 4; k++) {
 					equation[k].level++;
 				}
 				blt(&equation[i+1], &equation[i-1], (*np - (i - 1)) * sizeof(token_type));
 				*np += 2;
-				equation[i].level = level;
-				equation[i].kind = OPERATOR;
-				equation[i].token.operatr = TIMES;
 				equation[i-1].level = level;
 				equation[i-1].kind = CONSTANT;
 				equation[i-1].token.constant = d1;
+				equation[i].level = level;
+				equation[i].kind = OPERATOR;
+				equation[i].token.operatr = TIMES;
 				modified = true;
-				i += 2;
+				i += 4;
 			}
 		}
 	}
@@ -1012,6 +1123,22 @@ int		*np;
 	return modified;
 }
 
+double
+fixed_fmod(k1, k2)
+double	k1, k2;
+{
+	double	d;
+
+	if (k2 == 0 || !isfinite(k1) || !isfinite(k2) || (fmod(k1, 1.0) == 0 && fmod(k2, 1.0) == 0)) {
+		k1 = fmod(k1, k2);
+	} else {
+		/* fmod() doesn't work well with fractional amounts. */
+		/* Another way to get the remainder of division: */
+		k1 = modf(k1 / k2, &d) * k2;
+	}
+	return k1;
+}
+
 /*
  * Combine two or more constants on the same level of parentheses.
  * If "iflag" is false, don't produce imaginary numbers.
@@ -1100,14 +1227,9 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 		break;
 	case MODULUS:
 		if (k2 == 0) {
-			warning(_("Modulo 0 encountered, might be considered undefined."));
+			warning(_("Modulo 0 encountered."));
 		}
-#if	false	/* If fmod() always works.  It doesn't. */
-		*k1p = fmod(*k1p, k2);
-#else
-		/* Another way to get the remainder of division: */
-		*k1p = modf(*k1p / k2, &d) * k2;
-#endif
+		*k1p = fixed_fmod(*k1p, k2);
 		if (modulus_mode && *k1p < 0.0) {
 			*k1p += fabs(k2);	/* make positive */
 		}
@@ -1141,17 +1263,33 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 				}
 			}
 		}
+#if	1
+		if (errno == ERANGE) {	/* preserve overflowed powers rather than aborting with an error message */
+			domain_check = false;
+			return false;
+		}
+#endif
 		check_err();
 		if (domain_check)
 			*k1p = d;
 		break;
 	case FACTORIAL:
-#if	true	/* set this to false if lgamma() doesn't exist */
+#if	_XOPEN_SOURCE >= 600 || _ISOC99_SOURCE || __USE_ISOC99 || MINGW
+		/* Use true gamma for factorial if available, it is the nicest gamma() function. */
+		d = tgamma(*k1p + 1.0);
+		if (errno) {	/* don't evaluate if overflow */
+			return false;
+		}
+#else
+#if	1	/* Set this to 0 if lgamma() (logarithm of gamma) doesn't exist. */
+#warning "Using lgamma()."
+		/* Not threadsafe, but neither is most of Mathomatic with its global static data areas. */
 		d = exp(lgamma(*k1p + 1.0)) * signgam;
 		if (errno) {	/* don't evaluate if overflow */
 			return false;
 		}
 #else
+		/* calculate integer factorial if gamma() functions don't exist */
 		if (*k1p > 170.0 || *k1p < 0.0 || fmod(*k1p, 1.0) != 0.0) {
 			return false;
 		}
@@ -1159,6 +1297,7 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 		for (d1 = 2.0; d1 <= *k1p; d1 += 1.0) {
 			d *= d1;
 		}
+#endif
 #endif
 		*k1p = d;
 		break;
@@ -1243,11 +1382,34 @@ beginning:
 					d3 = pow(-d1, d2);
 					check_err();
 					if (!always_positive(denominator)) {
-						if (!always_positive(numerator)) {
-							d3 = -d3;
+						if (*np + 2 > n_tokens) {
+							error_huge();
 						}
-						d1 = d3;
-						goto not_imaginary;
+						blt(&equation[loc1+2], &equation[loc1], (*np - loc1) * sizeof(token_type));
+						*np += 2;
+						equation[loc1].level = level + 1;
+						equation[loc1].kind = CONSTANT;
+						equation[loc1].token.constant = -d1;
+						loc1++;
+						equation[loc1].level = level + 1;
+						equation[loc1].kind = OPERATOR;
+						equation[loc1].token.operatr = POWER;
+						equation[loc].level = level + 1;
+						equation[loc].kind = CONSTANT;
+						equation[loc].token.constant = d2;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = OPERATOR;
+						equation[loc].token.operatr = TIMES;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = CONSTANT;
+						if (always_positive(numerator)) {
+							equation[loc].token.constant = 1.0;
+						} else {
+							equation[loc].token.constant = -1.0;
+						}
+						return true;
 					}
 					if (!iflag)
 						return modified;
@@ -1298,7 +1460,6 @@ beginning:
 					}
 					return true;
 				} else {
-not_imaginary:
 					equation[loc1].token.constant = d1;
 					modified = true;
 					domain_check = false;
@@ -1446,7 +1607,7 @@ int		*np;		/* pointer to length of equation side */
 				}
 			}
 			if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
-/* Move any constant to the beginning of a multiplicative subexpression. */
+/* Move any constant to the beginning of a multiplicative sub-expression. */
 				d = (p1 + 1)->token.constant;
 				for (p2 = p1 - 1; p2 > equation; p2--) {
 					if ((p2 - 1)->level < level)
@@ -1465,11 +1626,10 @@ int		*np;		/* pointer to length of equation side */
 				(p2 + 1)->token.operatr = TIMES;
 				if (p2 > equation) {
 					p1 = p2 - 1;
-					continue;
 				} else {
 					p1 = equation + 1;
-					continue;
 				}
+				continue;
 			}
 			break;
 		case DIVIDE:
@@ -1512,6 +1672,29 @@ int		*np;		/* pointer to length of equation side */
 				modified = true;
 				continue;
 			}
+			if (p1->token.operatr == MODULUS) {
+				if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
+					d = fabs((p1 + 1)->token.constant);
+					if (d > epsilon && fmod(1.0 / d, 1.0) == 0.0) {
+						for (p2 = p1 - 1; p2 > equation; p2--) {
+							if ((p2 - 1)->level < level)
+								break;
+						}
+						if (is_integer_expr(p2, p1 - p2)) {
+							blt(p2, p1 + 1, (char *) ep - (char *) (p1 + 1));
+							*np -= (p1 + 1) - p2;
+							p2->token.constant = 0.0;
+							if (p2 > equation) {
+								p1 = p2 - 1;
+							} else {
+								p1 = equation + 1;
+							}
+							modified = true;
+							continue;
+						}
+					}
+				}
+			}
 			break;
 		case POWER:
 			if (p2->level == level && p2->kind == CONSTANT) {
@@ -1575,7 +1758,12 @@ int		*diff_signp;	/* different sign flag pointer */
 {
 	int	l1, l2;
 
-	/* First, find the proper ground levels of parentheses for the two sub-expressions. */
+	if (n1 == 0 || n2 == 0 || (((n1 > n2) ? ((n1 + 1) / (n2 + 1)) : ((n2 + 1) / (n1 + 1))) > 3)) {
+		/* expressions are grossly different in size, no need to compare, they are different */
+		*diff_signp = false;
+		return false;
+	}
+	/* Find the proper ground levels of parentheses for the two sub-expressions: */
 	l1 = min_level(p1, n1);
 	l2 = min_level(p2, n2);
 	return compare_recurse(p1, n1, l1, p2, n2, l2, diff_signp);
@@ -1591,6 +1779,8 @@ int		n1, l1;
 token_type	*p2;
 int		n2, l2, *diff_signp;
 {
+#define	compare_epsilon	epsilon
+
 	token_type	*pv1, *ep1, *ep2;
 	int		i, j;
 	int		len;
@@ -1598,10 +1788,9 @@ int		n2, l2, *diff_signp;
 	int		oc2;				/* operand count 2 */
 	token_type	*opa2[MAX_COMPARE_TERMS];	/* operand pointer array 2 */
 	char		used[MAX_COMPARE_TERMS];	/* operand used flag array 2 */
-	int		last_op1, op1, op2;
-	int		diff_op;
+	int		last_op1, op1 = 0, op2 = 0;
+	int		diff_op = false;
 	double		d1, c1, c2;
-	double		compare_epsilon = epsilon;
 
 	*diff_signp = false;
 	if (n1 == 1 && n2 == 1) {
@@ -1635,27 +1824,30 @@ int		n2, l2, *diff_signp;
 			}
 			break;
 		case OPERATOR:
+			error_bug("Programming error in call to compare_recurse().");
 			break;
 		}
 		return false;
 	}
+#if	DEBUG
+	if ((n1 & 1) != 1 || (n2 & 1) != 1) {
+		error_bug("Programming error in call to compare_recurse().");
+	}
+#endif
 	ep1 = &p1[n1];
 	ep2 = &p2[n2];
-	op1 = 0;
 	for (pv1 = p1 + 1; pv1 < ep1; pv1 += 2) {
 		if (pv1->level == l1) {
 			op1 = pv1->token.operatr;
 			break;
 		}
 	}
-	op2 = 0;
 	for (pv1 = p2 + 1; pv1 < ep2; pv1 += 2) {
 		if (pv1->level == l2) {
 			op2 = pv1->token.operatr;
 			break;
 		}
 	}
-	diff_op = false;
 	if (op2 == 0) {
 		if (op1 != TIMES && op1 != DIVIDE)
 			return false;
@@ -1832,6 +2024,8 @@ int		*np;
 
 	for (i = 1; i < *np; i += 2) {
 		level = equation[i].level;
+#if	1	/* This code makes complex number calculations give different (maybe wrong) results. */
+		/* It converts x/i to x*-1*i.  Might be better to do this only when symb_flag is true. */
 		if (equation[i].token.operatr == DIVIDE) {
 			if (equation[i+1].level == level
 			    && equation[i+1].kind == VARIABLE
@@ -1855,6 +2049,7 @@ int		*np;
 				modified = true;
 			}
 		}
+#endif
 		if (equation[i].token.operatr == POWER
 		    && equation[i+1].level == level
 		    && equation[i+1].kind == CONSTANT) {
@@ -2094,7 +2289,7 @@ int		i1, i2;
 }
 
 /*
- * Recursively check and possibly reorder a "level" subexpression
+ * Recursively check and possibly reorder a "level" sub-expression
  * which starts at "loc" in an equation side.
  */
 static int
@@ -2106,13 +2301,13 @@ int		*np, loc, level;
 	int	op = 0;
 	int	modified = false;
 
-/* check passed subexpression for validity: */
+/* check passed sub-expression for validity: */
 	if ((loc & 1) != 0) {
 		goto corrupt;
 	}
 	for (i = loc; i < *np;) {
 		if (equation[i].level < level) {
-			if (equation[i].kind != OPERATOR) {
+			if (equation[i].kind != OPERATOR || equation[i].level < 1) {
 				goto corrupt;
 			}
 			break;
@@ -2124,11 +2319,8 @@ int		*np, loc, level;
 				;
 			continue;
 		} else {
-			if (((i & 1) == 1) != (equation[i].kind == OPERATOR)) {
-				goto corrupt;
-			}
 			if (equation[i].kind == OPERATOR) {
-				if (!equation[i].token.operatr) {
+				if ((i & 1) == 0 || equation[i].token.operatr == 0) {
 					goto corrupt;
 				}
 				if (op) {
@@ -2148,14 +2340,17 @@ int		*np, loc, level;
 				} else {
 					op = equation[i].token.operatr;
 				}
+			} else {
+				if ((i & 1) != 0)
+					goto corrupt;
 			}
 		}
 		i++;
 	}
-	if ((i & 1) != 1) {	/* "i" should be at the end of the current subexpression */
+	if ((i & 1) == 0) {	/* "i" should be at the end of the current sub-expression */
 		goto corrupt;
 	}
-/* reorder subexpression if needed: */
+/* reorder sub-expression if needed: */
 	switch (op) {
 	case PLUS:
 	case MINUS:
@@ -2228,6 +2423,11 @@ int		*np;
 be_thorough:
 		if (i >= *np)
 			break;
+#if	DEBUG
+		if (equation[i].kind != OPERATOR) {
+			error_bug("Bug in rationalize().");
+		}
+#endif
 		if (equation[i].token.operatr == DIVIDE) {
 			div_level = equation[i].level;
 			count = 0;
@@ -2318,8 +2518,10 @@ do_again:
 						equation[k1].token.operatr = DIVIDE;
 						k1++;
 						blt(&equation[k1], scratch, k * sizeof(token_type));
-						i = k1 + k;
-						debug_string(1, "Square roots in denominator rationalized.");
+						k1 += k;
+						debug_string(1, "Square roots in denominator rationalized:");
+						side_debug(1, &equation[i+1], k1 - (i + 1));
+						i = k1;
 						modified = true;
 						goto be_thorough;
 					}
