@@ -1,7 +1,7 @@
 /*
  * Group and combine algebraic fractions for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -21,15 +21,15 @@
 
 #include "includes.h"
 
-static int	sf_recurse();
-static int	sf_sub();
+static int sf_recurse(token_type *equation, int *np, int loc, int level, int start_flag);
+static int sf_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level, int check_flag);
 
 static void
 group_recurse(equation, np, loc, level)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 int		loc;		/* starting location within equation side */
-int		level;		/* current level of parentheses within the subexpression starting at "loc" */
+int		level;		/* current level of parentheses within the sub-expression starting at "loc" */
 {
 	int		i;
 	int		len;
@@ -82,7 +82,7 @@ int		level;		/* current level of parentheses within the subexpression starting a
 	if (group_flag) {
 		for (i = di + 1; i < edi; i++) {
 			if (equation[i].level == level && equation[i].kind == OPERATOR) {
-#if	false
+#if	1
 				if (equation[i].token.operatr != DIVIDE) {
 					error_bug("Bug in group_recurse().");
 				}
@@ -120,16 +120,19 @@ fractions_and_group(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	rv;
+	int	rv = false;
 
 	elim_loop(equation, np);
-	rv = make_fractions(equation, np);
+	if (fractions_display) {
+		rv = make_fractions(equation, np);
+	}
 	group_proc(equation, np);
 	return rv;
 }
 
 /*
  * This function is the guts of the display command.
+ * Makes an equation space ready for display.
  *
  * Return true if any fractions were created.
  */
@@ -234,7 +237,18 @@ sf_again:
 		for (k = j + 1; k < *np && equation[k].level > level; k += 2)
 			;
 		len2 = k - j;
+#if	0
+		side_debug(0, &equation[i], len1);
+		side_debug(0, &equation[j], len2);
+#endif
 		if (sf_sub(equation, np, loc, i, len1, j, len2, level + 1, start_flag == 2)) {
+#if	0
+			int junk;
+			debug_string(0, "super_factor() successful:");
+			for (junk = 1; (loc + junk) < *np && equation[loc+junk].level > level; junk += 2)
+				;
+			side_debug(0, &equation[loc], junk);
+#endif
 			modified = true;
 			goto sf_again;
 		}
@@ -255,6 +269,7 @@ int		*np, loc, i1, n1, i2, n2, level, check_flag;
 	token_type	*p1, *p2;
 	int		np1, np2;
 	int		div_flag1 = false, div_flag2 = false;
+	int		rv;
 
 	e1 = i1 + n1;
 	e2 = i2 + n2;
@@ -293,19 +308,19 @@ int		*np, loc, i1, n1, i2, n2, level, check_flag;
 	if (!div_flag1 && !div_flag2)
 		return false;
 	if (check_flag && div_flag1 && div_flag2) {
-		if (poly2_gcd(&equation[b1], i - b1, &equation[b2], j - b2, 0L)) {
+		if ((rv = poly2_gcd(&equation[b1], i - b1, &equation[b2], j - b2, 0L, true)) > 0) {
 			p1 = tlhs;
 			np1 = n_tlhs;
 			p2 = trhs;
 			np2 = n_trhs;
-			goto do_gcd;
+			goto do_gcd_super;
 		}
-		if (poly2_gcd(&equation[b2], j - b2, &equation[b1], i - b1, 0L)) {
+		if (rv == 0 && poly2_gcd(&equation[b2], j - b2, &equation[b1], i - b1, 0L, true) > 0) {
 			p1 = trhs;
 			np1 = n_trhs;
 			p2 = tlhs;
 			np2 = n_tlhs;
-			goto do_gcd;
+			goto do_gcd_super;
 		}
 	}
         if (n1 + n2 + (i - b1) + (j - b2) + 8 > n_tokens) {
@@ -401,7 +416,7 @@ end_mess:
 	blt(&equation[i1], scratch, len * sizeof(token_type));
 	return true;
 
-do_gcd:
+do_gcd_super:
 	if (5 - i1 + e1 + (2*np2) + b2 - i2 + e2 - j + np1 > n_tokens) {
 		error_huge();
 	}
