@@ -1,7 +1,7 @@
 /*
  * Mathomatic unfactorizing (expanding) routines.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public 
@@ -21,12 +21,12 @@
 
 #include "includes.h"
 
-static int	unf_sub();
+static int unf_sub(token_type *equation, int *np, int b1, int loc, int e1, int level, int ii);
 
 /*
  * Unfactor times and divide only (products of sums) and simplify.
  *
- * Return true if equation side was modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_tsimp(equation, np)
@@ -48,42 +48,42 @@ int		*np;
  * Unfactor power only.
  * (a * b)^c -> a^c * b^c
  *
- * Return true if equation side is modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_power(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	rv;
+	int	count = -1;
 
-	organize(equation, np);
-	rv = sub_ufactor(equation, np, 2);
-	if (rv) {
+	do {
 		organize(equation, np);
-	}
-	return rv;
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 2));
+	return count;
 }
 
 /*
  * Unfactor power only.
  * a^(b + c) -> a^b * a^c
  *
- * Return true if equation side is modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_pplus(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	rv;
+	int	count = -1;
 
-	organize(equation, np);
-	rv = sub_ufactor(equation, np, 4);
-	if (rv) {
+	do {
 		organize(equation, np);
-	}
-	return rv;
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 4));
+	return count;
 }
 
 /*
@@ -113,10 +113,13 @@ uf_repeat(equation, np)
 token_type	*equation;
 int		*np;
 {
-	organize(equation, np);
-	if (sub_ufactor(equation, np, 6)) {
+	int	count = -1;
+
+	do {
 		organize(equation, np);
-	}
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 6));
 	patch_root_div(equation, np);
 }
 
@@ -130,10 +133,13 @@ uf_repeat_always(equation, np)
 token_type	*equation;
 int		*np;
 {
-	organize(equation, np);
-	if (sub_ufactor(equation, np, 8)) {
+	int	count = -1;
+
+	do {
 		organize(equation, np);
-	}
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 8));
 }
 
 /*
@@ -207,7 +213,7 @@ int		*np;
  * (a + b)*c -> a*c + b*c
  * If partial_flag is true, don't expand (a+b)/(c+d) nor (a+b)/c.
  *
- * Return true if equation side was modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_times(equation, np)
@@ -219,8 +225,8 @@ int		*np;
 
 	do {
 		organize(equation, np);
-		if (partial_flag) {
-			reorder(equation, np);
+		if (reorder(equation, np)) {
+			organize(equation, np);
 		}
 		group_proc(equation, np);
 		if (partial_flag) {
@@ -570,7 +576,7 @@ int		*np;
 	return modified;
 }
 
-#if	false	/* the following is not currently used */
+#if	0	/* the following is not currently used */
 /*
  * Convert a/(x^y) to a*(1/x)^y.
  * Return true if equation side is modified.
